#!/usr/bin/env python

#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for `opencue.wrappers.owner`."""

from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest

import mock

from opencue_proto import host_pb2
import opencue.wrappers.owner


TEST_DEED_ID = 'ddd-dd-dddd'
TEST_HOST_ID = 'hhh-hh-hhhh'
TEST_OWNER_ID = 'ooo-oo-oooo'
TEST_OWNER_NAME = 'testOwner'
TEST_SHOW_NAME = 'testShow'


@mock.patch('opencue.cuebot.Cuebot.getStub')
class OwnerTests(unittest.TestCase):

    def testDelete(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Delete.return_value = host_pb2.OwnerDeleteResponse()
        getStubMock.return_value = stubMock

        owner = opencue.wrappers.owner.Owner(
            host_pb2.Owner(id=TEST_OWNER_ID, name=TEST_OWNER_NAME))
        owner.delete()

        stubMock.Delete.assert_called_with(
            host_pb2.OwnerDeleteRequest(owner=owner.data), timeout=mock.ANY)

    def testGetDeeds(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetDeeds.return_value = host_pb2.OwnerGetDeedsResponse(
            deeds=host_pb2.DeedSeq(deeds=[host_pb2.Deed(id=TEST_DEED_ID)]))
        getStubMock.return_value = stubMock

        owner = opencue.wrappers.owner.Owner(
            host_pb2.Owner(id=TEST_OWNER_ID, name=TEST_OWNER_NAME))
        deeds = owner.getDeeds()

        stubMock.GetDeeds.assert_called_with(
            host_pb2.OwnerGetDeedsRequest(owner=owner.data), timeout=mock.ANY)
        self.assertEqual(len(deeds), 1)
        self.assertEqual(deeds[0].id(), TEST_DEED_ID)

    def testGetHosts(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetHosts.return_value = host_pb2.OwnerGetHostsResponse(
            hosts=host_pb2.HostSeq(hosts=[host_pb2.Host(id=TEST_HOST_ID)]))
        getStubMock.return_value = stubMock

        owner = opencue.wrappers.owner.Owner(
            host_pb2.Owner(id=TEST_OWNER_ID, name=TEST_OWNER_NAME))
        hosts = owner.getHosts()

        stubMock.GetHosts.assert_called_with(
            host_pb2.OwnerGetHostsRequest(owner=owner.data), timeout=mock.ANY)
        self.assertEqual(len(hosts), 1)
        self.assertEqual(hosts[0].id(), TEST_HOST_ID)

    def testGetOwner(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetOwner.return_value = host_pb2.OwnerGetOwnerResponse(
            owner=host_pb2.Owner(id=TEST_OWNER_ID, name=TEST_OWNER_NAME))
        getStubMock.return_value = stubMock

        owner = opencue.wrappers.owner.Owner()
        response = owner.getOwner(TEST_OWNER_NAME)

        stubMock.GetOwner.assert_called_with(
            host_pb2.OwnerGetOwnerRequest(name=TEST_OWNER_NAME), timeout=mock.ANY)
        self.assertEqual(response.id(), TEST_OWNER_ID)
        self.assertEqual(response.name(), TEST_OWNER_NAME)

    def testSetShow(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetShow.return_value = host_pb2.OwnerSetShowResponse()
        getStubMock.return_value = stubMock

        owner = opencue.wrappers.owner.Owner(
            host_pb2.Owner(id=TEST_OWNER_ID, name=TEST_OWNER_NAME))
        owner.setShow(TEST_SHOW_NAME)

        stubMock.SetShow.assert_called_with(
            host_pb2.OwnerSetShowRequest(owner=owner.data, show=TEST_SHOW_NAME),
            timeout=mock.ANY)

    def testTakeOwnership(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.TakeOwnership.return_value = host_pb2.OwnerTakeOwnershipResponse()
        getStubMock.return_value = stubMock

        owner = opencue.wrappers.owner.Owner(
            host_pb2.Owner(id=TEST_OWNER_ID, name=TEST_OWNER_NAME))
        owner.takeOwnership(TEST_HOST_ID)

        stubMock.TakeOwnership.assert_called_with(
            host_pb2.OwnerTakeOwnershipRequest(owner=owner.data, host=TEST_HOST_ID),
            timeout=mock.ANY)


if __name__ == '__main__':
    unittest.main()



#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Module for classes related to comments."""

from opencue_proto import comment_pb2
from opencue.cuebot import Cuebot


class Comment(object):
    """This class contains the grpc implementation related to a Comment."""

    def __init__(self, comment=None):
        self.data = comment
        self.stub = Cuebot.getStub('comment')

    def delete(self):
        """Deletes this comment."""
        self.stub.Delete(
            comment_pb2.CommentDeleteRequest(comment=self.data), timeout=Cuebot.Timeout)

    def save(self):
        """Saves the current comment values."""
        self.stub.Save(comment_pb2.CommentSaveRequest(comment=self.data), timeout=Cuebot.Timeout)

    def message(self):
        """Returns the message of the comment.

        :rtype:  str
        :return: comment message
        """
        return self.data.message

    def subject(self):
        """Returns the subject of the comment.

        :rtype:  str
        :return: comment subject
        """
        return self.data.subject

    def user(self):
        """Returns the username of the user who submitted the comment.

        :rtype:  str
        :return: username of the commenter
        """
        return self.data.user

    def timestamp(self):
        """Returns the timestamp for the comment as an epoch.

        :rtype:  int
        :return: the time the comment was submitted as an epoch
        """
        return self.data.timestamp

    def setMessage(self, message):
        """Sets a new message for the comment.

        :type  message: str
        :param message: a new message
        """
        self.data.message = message

    def setSubject(self, subject):
        """Sets a new subject for the comment.

        :type  subject: str
        :param subject: a new subject
        """
        self.data.subject = subject



#!/usr/bin/env python

#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for `opencue.wrappers.comment`."""

from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest

import mock

from opencue_proto import comment_pb2
import opencue.wrappers.comment


TEST_COMMENT_MESSAGE = "hi, I'm a message"


@mock.patch('opencue.cuebot.Cuebot.getStub')
class CommentTests(unittest.TestCase):

    def testDelete(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Delete.return_value = comment_pb2.CommentDeleteResponse()
        getStubMock.return_value = stubMock

        comment = opencue.wrappers.comment.Comment(
            comment_pb2.Comment(message=TEST_COMMENT_MESSAGE))
        comment.delete()

        stubMock.Delete.assert_called_with(
            comment_pb2.CommentDeleteRequest(comment=comment.data), timeout=mock.ANY)

    def testSave(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Save.return_value = comment_pb2.CommentSaveResponse()
        getStubMock.return_value = stubMock

        comment = opencue.wrappers.comment.Comment(
            comment_pb2.Comment(message=TEST_COMMENT_MESSAGE))
        comment.save()

        stubMock.Save.assert_called_with(
            comment_pb2.CommentSaveRequest(comment=comment.data), timeout=mock.ANY)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for `opencue.wrappers.subscription`."""

from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest

import mock

from opencue_proto import subscription_pb2
import opencue.wrappers.subscription


TEST_SUBSCRIPTION_ID = 'aaaa-aaa-aaaa'
TEST_SUBSCRIPTION_NAME = 'testSubscription'


@mock.patch('opencue.cuebot.Cuebot.getStub')
class SubscriptionTests(unittest.TestCase):

    def testDelete(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Delete.return_value = subscription_pb2.SubscriptionDeleteResponse()
        getStubMock.return_value = stubMock

        subscription = opencue.wrappers.subscription.Subscription(
            subscription_pb2.Subscription(name=TEST_SUBSCRIPTION_NAME))
        subscription.delete()

        stubMock.Delete.assert_called_with(
            subscription_pb2.SubscriptionDeleteRequest(subscription=subscription.data),
            timeout=mock.ANY)

    def testFind(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Find.return_value = subscription_pb2.SubscriptionFindResponse(
            subscription=subscription_pb2.Subscription(name=TEST_SUBSCRIPTION_NAME))
        getStubMock.return_value = stubMock

        expected = opencue.wrappers.subscription.Subscription(
            subscription_pb2.Subscription(name=TEST_SUBSCRIPTION_NAME))
        wrapper = opencue.wrappers.subscription.Subscription()
        subscription = wrapper.find(name=TEST_SUBSCRIPTION_NAME)

        stubMock.Find.assert_called_with(
            subscription_pb2.SubscriptionFindRequest(name=TEST_SUBSCRIPTION_NAME),
            timeout=mock.ANY)
        self.assertEqual(expected.name(), subscription.name())

    def testGet(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Get.return_value = subscription_pb2.SubscriptionGetResponse(
            subscription=subscription_pb2.Subscription(id=TEST_SUBSCRIPTION_ID,
                                                       name=TEST_SUBSCRIPTION_NAME))
        getStubMock.return_value = stubMock

        wrapper = opencue.wrappers.subscription.Subscription()
        subscription = wrapper.get(TEST_SUBSCRIPTION_ID)

        stubMock.Get.assert_called_with(
            subscription_pb2.SubscriptionGetRequest(id=TEST_SUBSCRIPTION_ID),
            timeout=mock.ANY)
        self.assertEqual(subscription.id(), TEST_SUBSCRIPTION_ID)

    def testSetBurst(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetBurst.return_value = subscription_pb2.SubscriptionSetBurstResponse()
        getStubMock.return_value = stubMock

        subscription = opencue.wrappers.subscription.Subscription(
            subscription_pb2.Subscription(name='testSubscription'))
        subscription.setBurst(15)

        stubMock.SetBurst.assert_called_with(
            subscription_pb2.SubscriptionSetBurstRequest(subscription=subscription.data, burst=15),
            timeout=mock.ANY)

    def testSetSize(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetSize.return_value = subscription_pb2.SubscriptionSetSizeResponse()
        getStubMock.return_value = stubMock

        subscription = opencue.wrappers.subscription.Subscription(
            subscription_pb2.Subscription(name='testSubscription'))
        subscription.setSize(16)

        stubMock.SetSize.assert_called_with(
            subscription_pb2.SubscriptionSetSizeRequest(
                subscription=subscription.data, new_size=16),
            timeout=mock.ANY)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for `opencue.wrappers.allocation`."""

from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest

import mock

from opencue_proto import facility_pb2
from opencue_proto import host_pb2
from opencue_proto import subscription_pb2
import opencue.wrappers.allocation
import opencue.wrappers.host


TEST_ALLOC_NAME = 'test_allocation'
TEST_ALLOC_TAG = 'test_tag'
TEST_HOST_NAME = 'test_host'
TEST_HOST_ID = 'hhh-hhhh-hhh'
TEST_SUBSCRIPTION_NAME = 'test_subscription'


@mock.patch('opencue.cuebot.Cuebot.getStub')
class AllocationTests(unittest.TestCase):

    def testDelete(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Delete.return_value = facility_pb2.AllocDeleteResponse()
        getStubMock.return_value = stubMock

        alloc = opencue.wrappers.allocation.Allocation(
            facility_pb2.Allocation(name=TEST_ALLOC_NAME))
        alloc.delete()

        stubMock.Delete.assert_called_with(
            facility_pb2.AllocDeleteRequest(allocation=alloc.data), timeout=mock.ANY)

    def testGetHosts(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetHosts.return_value = facility_pb2.AllocGetHostsResponse(
            hosts=host_pb2.HostSeq(
                hosts=[host_pb2.Host(name=TEST_HOST_NAME)]))
        getStubMock.return_value = stubMock

        alloc = opencue.wrappers.allocation.Allocation(
            facility_pb2.Allocation(name=TEST_ALLOC_NAME))
        hosts = alloc.getHosts()

        stubMock.GetHosts.assert_called_with(
            facility_pb2.AllocGetHostsRequest(allocation=alloc.data), timeout=mock.ANY)
        self.assertEqual(len(hosts), 1)
        self.assertEqual(hosts[0].name(), TEST_HOST_NAME)

    def testGetSubscriptions(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetSubscriptions.return_value = facility_pb2.AllocGetSubscriptionsResponse(
            subscriptions=subscription_pb2.SubscriptionSeq(
                subscriptions=[subscription_pb2.Subscription(name=TEST_SUBSCRIPTION_NAME)]))
        getStubMock.return_value = stubMock

        alloc = opencue.wrappers.allocation.Allocation(
            facility_pb2.Allocation(name=TEST_ALLOC_NAME))
        subscriptions = alloc.getSubscriptions()

        stubMock.GetSubscriptions.assert_called_with(
            facility_pb2.AllocGetSubscriptionsRequest(allocation=alloc.data), timeout=mock.ANY)
        self.assertEqual(len(subscriptions), 1)
        self.assertEqual(subscriptions[0].name(), TEST_SUBSCRIPTION_NAME)

    def testReparentHosts(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.ReparentHosts.return_value = facility_pb2.AllocReparentHostsResponse()
        getStubMock.return_value = stubMock

        alloc = opencue.wrappers.allocation.Allocation(
            facility_pb2.Allocation(name=TEST_ALLOC_NAME))
        hosts = [opencue.wrappers.host.Host(host_pb2.Host(name=TEST_HOST_NAME))]
        alloc.reparentHosts(hosts)

        stubMock.ReparentHosts.assert_called_with(
            facility_pb2.AllocReparentHostsRequest(
                allocation=alloc.data,
                hosts=host_pb2.HostSeq(hosts=[host.data for host in hosts])),
            timeout=mock.ANY)

    def testReparentHostIds(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.ReparentHosts.return_value = facility_pb2.AllocReparentHostsResponse()
        getStubMock.return_value = stubMock

        alloc = opencue.wrappers.allocation.Allocation(
            facility_pb2.Allocation(name=TEST_ALLOC_NAME))
        hostIds = [TEST_HOST_ID]
        alloc.reparentHostIds(hostIds)
        hosts = [host_pb2.Host(id=TEST_HOST_ID)]

        stubMock.ReparentHosts.assert_called_with(
            facility_pb2.AllocReparentHostsRequest(
                allocation=alloc.data, hosts=host_pb2.HostSeq(hosts=hosts)), timeout=mock.ANY)

    def testSetName(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetName.return_value = facility_pb2.AllocSetNameResponse()
        getStubMock.return_value = stubMock

        alloc = opencue.wrappers.allocation.Allocation(facility_pb2.Allocation())
        alloc.setName(TEST_ALLOC_NAME)

        stubMock.SetName.assert_called_with(
            facility_pb2.AllocSetNameRequest(allocation=alloc.data, name=TEST_ALLOC_NAME),
            timeout=mock.ANY)

    def setTag(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetTag.return_value = facility_pb2.AllocSetTagResponse()
        getStubMock.return_value = stubMock

        alloc = opencue.wrappers.allocation.Allocation(facility_pb2.Allocation())
        alloc.setTag(TEST_ALLOC_TAG)

        stubMock.SetTag.assert_called_with(
            facility_pb2.AllocSetTagRequest(allocation=alloc.data, tag=TEST_ALLOC_TAG),
            timeout=mock.ANY)


if __name__ == '__main__':
    unittest.main()



#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Utility methods used throughout the opencue module."""

from __future__ import absolute_import
from __future__ import print_function
from __future__ import division

from builtins import str
import functools
import future.utils
import logging
import os
import time

import grpc

import opencue

logger = logging.getLogger('opencue')


def grpcExceptionParser(grpcFunc):
    """Decorator to wrap functions making GRPC calls.
    Attempts to throw the appropriate exception based on grpc status code."""
    def _decorator(*args, **kwargs):
        triesRemaining = opencue.exception.getRetryCount() + 1
        while triesRemaining > 0:
            triesRemaining -= 1
            try:
                return grpcFunc(*args, **kwargs)
            except grpc.RpcError as exc:
                # pylint: disable=no-member
                code = exc.code()
                details = exc.details() or "No details found. Check server logs."
                # pylint: enable=no-member
                exception = opencue.exception.EXCEPTION_MAP.get(code)
                if exception:
                    if exception.retryable and triesRemaining >= 1:
                        logger.warning(exception.retryMsg)
                        time.sleep(exception.retryBackoff)
                    else:
                        future.utils.raise_with_traceback(
                            exception(exception.failMsg.format(details=details)))
                else:
                    future.utils.raise_with_traceback(opencue.exception.CueException(
                        "Encountered a server error. {code} : {details}".format(
                            code=code, details=details)))

    return functools.wraps(grpcFunc)(_decorator)


# pylint: disable=redefined-builtin
def id(value):
    """extract(entity)
    extracts a string unique ID from a opencue entity or
    list of opencue entities.
    """
    def _extract(item):
        try:
            return item.id()
        # pylint: disable=bare-except
        except:
            pass
        return item

    if isinstance(value, (tuple, list, set)):
        return [_extract(v) for v in value]
    return _extract(value)


@grpcExceptionParser
def proxy(idOrObject, cls):
    """Helper function for getting proto objects back from Cuebot.

    :type  idOrObject: str, list<str>, protobuf Message, list<protobuf Message>
    :param idOrObject: The id/item, or list of ids/items to look up
    :type cls: str
    :param cls: The Name of the protobuf message class to use.
    :rtype:  protobuf Message or list
    :return: Cue object or list of objects"""
    def _proxy(idString):
        proto = opencue.Cuebot.PROTO_MAP.get(cls.lower())
        if proto:
            requestor = getattr(proto, "{cls}Get{cls}Request".format(cls=cls))
            getMethod = getattr(opencue.Cuebot.getStub(cls.lower()), "Get{}".format(cls))
            return getMethod(requestor(id=idString))
        raise AttributeError('Could not find a proto for {}'.format(cls))

    def _proxies(entities):
        messages = []
        for item in entities:
            if hasattr(item, 'id'):
                messages.append(_proxy(item.id))
            else:
                messages.append(_proxy(item))
        return messages

    if hasattr(idOrObject, 'id'):
        return _proxy(idOrObject.id)
    if isinstance(idOrObject, str):
        return _proxy(idOrObject)
    return _proxies(idOrObject)


def rep(entity):
    """rep(entity)
    Extracts a string representation of a opencue entity"""
    try:
        return entity.name
    # pylint: disable=bare-except
    except:
        return str(entity)


def logPath(job, frame=None):
    """logPath(job, frame=None)
        Extracts the log path from a job or a job/frame
    """
    if frame:
        return os.path.join(job.data.log_dir, "%s.%s.rqlog" % (job.data.name, frame.data.name))
    return job.data.log_dir



#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Module for classes related to services."""

import grpc

from opencue_proto import service_pb2
from opencue.cuebot import Cuebot


class Service(object):
    """This class contains the grpc implementation related to a Service."""

    def __init__(self, service=None):
        self.data = service or service_pb2.Service()
        self.stub = Cuebot.getStub('service')

    def create(self):
        """Creates a service in the database using the current instance data."""
        # min_memory_increase has to be greater than 0.
        if self.data.min_memory_increase <= 0:
            raise ValueError("Minimum memory increase must be > 0")

        response = self.stub.CreateService(
            service_pb2.ServiceCreateServiceRequest(data=self.data),
            timeout=Cuebot.Timeout)
        return Service(response.service)

    def delete(self):
        """Deletes the service."""
        return self.stub.Delete(
            service_pb2.ServiceDeleteRequest(service=self.data),
            timeout=Cuebot.Timeout)

    @staticmethod
    def getDefaultServices():
        """Returns the default services."""
        response = Cuebot.getStub('service').GetDefaultServices(
            service_pb2.ServiceGetDefaultServicesRequest(),
            timeout=Cuebot.Timeout)
        return [Service(data) for data in response.services.services]

    @staticmethod
    def getService(name):
        """Returns a service by name.

        :type  name: str
        :param name: service name to find
        """
        try:
            response = Cuebot.getStub('service').GetService(
                service_pb2.ServiceGetServiceRequest(name=name),
                timeout=Cuebot.Timeout)
        except grpc.RpcError as e:
            # pylint: disable=no-member
            if e.code() == grpc.StatusCode.NOT_FOUND:
                return None
            # pylint: enable=no-member
            raise e
        return Service(response.service)

    def update(self):
        """Updates the service database record with the current instance data."""
        # min_memory_increase has to be greater than 0.
        if self.data.min_memory_increase <= 0:
            raise ValueError("Minimum memory increase must be > 0")

        return self.stub.Update(
            service_pb2.ServiceUpdateRequest(service=self.data),
            timeout=Cuebot.Timeout)

    def id(self):
        """Returns the id of the service.

        :rtype:  str
        :return: service id
        """
        return self.data.id

    def name(self):
        """Returns the name of the service.

        :rtype:  str
        :return: service name
        """
        return self.data.name

    def setName(self, name):
        """Sets the service name.

        :type: string
        :param: new service name
        """
        self.data.name = name

    def threadable(self):
        """Returns whether the service is threadable.

        :rtype:  bool
        :return: whether service is threadable
        """
        return self.data.threadable

    def setThreadable(self, threadable):
        """Sets the threadable field of the service.

        :type:  bool
        :param: whether or not the service should be threadable
        """
        self.data.threadable = threadable

    def minCores(self):
        """Returns the minimum cores of the service.

        :rtype:  int
        :return: min cores
        """
        return self.data.min_cores

    def setMinCores(self, minCores):
        """Sets the minimum cores of the service.

        :type: int
        :param: new min cores
        """
        self.data.min_cores = minCores

    def maxCores(self):
        """Returns the maximum cores of the service.

        :rtype:  int
        :return: max cores
        """
        return self.data.max_cores

    def setMaxCores(self, maxCores):
        """Sets the maximum cores of the service.

        :type: int
        :param: new max cores
        """
        self.data.max_cores = maxCores

    def minMemory(self):
        """Returns the minimum memory of the service.

        :rtype:  int
        :return: min memory
        """
        return self.data.min_memory

    def setMinMemory(self, minMemory):
        """Sets the minimum memory field of the service.

        :type: int
        :param: new min memory
        """
        self.data.min_memory = minMemory

    def minGpus(self):
        """Returns the minimum gpus of the service.

        :rtype:  int
        :return: min gpus
        """
        return self.data.min_gpus

    def setMinGpus(self, minGpus):
        """Sets the minimum gpus of the service.

        :type: int
        :param: new min gpus
        """
        self.data.min_gpus = minGpus

    def maxGpus(self):
        """Returns the maximum gpus of the service.

        :rtype:  int
        :return: max gpus
        """
        return self.data.max_gpus

    def setMaxGpus(self, maxGpus):
        """Sets the maximum gpus of the service.

        :type: int
        :param: new max gpus
        """
        self.data.max_gpus = maxGpus

    def minGpuMemory(self):
        """Returns the minimum gpu memory of the service.

        :rtype:  int
        :return: min gpu memory
        """
        return self.data.min_gpu_memory

    def setMinGpuMemory(self, minGpuMemory):
        """Sets the minimum gpu memory of the service.

        :type: int
        :param: new min gpu memory
        """
        self.data.min_gpu_memory = minGpuMemory

    def tags(self):
        """Returns the list of tags for the service.

        :rtype:  list<string>
        :return: list of service tags
        """
        return self.data.tags

    def setTags(self, tags):
        """Clears and sets the service tags.

        :type:  list<string>
        :param: new list of service tags
        """
        self.data.tags[:] = tags

    def timeout(self):
        """Gets the default service timeout."""
        return self.data.timeout

    def setTimeout(self, timeout):
        """Sets the default service timeout."""
        self.data.timeout = timeout

    def timeoutLLU(self):
        """Gets the default service LLU timeout."""
        return self.data.timeout

    def setTimeoutLLU(self, timeout_llu):
        """Sets the default service LLU timeout."""
        self.data.timeout_llu = timeout_llu

    def minMemoryIncrease(self):
        """Gets the default service minimum memory increment"""
        return self.data.min_memory_increase

    def setMinMemoryIncrease(self, min_memory_increase):
        """Sets the default service minimum memory increment"""
        if min_memory_increase > 0:
            self.data.min_memory_increase = min_memory_increase
        else:
            raise ValueError("Minimum memory increase must be > 0")

class ServiceOverride(object):
    """Represents a display override of a service assigned to a show"""
    def __init__(self, serviceOverride=None):
        defaultServiceOverride = service_pb2.ServiceOverride()
        # pylint: disable=no-member
        self.id = defaultServiceOverride.id
        self.data = defaultServiceOverride.data
        if serviceOverride:
            self.id = serviceOverride.id
            self.data = serviceOverride.data or service_pb2.Service().data
        self.stub = Cuebot.getStub("serviceOverride")
        # pylint: enable=no-member

    def delete(self):
        """Remove service override"""
        self.stub.Delete(
            service_pb2.ServiceOverrideDeleteRequest(service=self.data),
            timeout=Cuebot.Timeout)

    def update(self):
        """Commit a ServiceOverride change to the database"""
        self.stub.Update(
            service_pb2.ServiceOverrideUpdateRequest(service=self.data),
            timeout=Cuebot.Timeout)



#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Top level of the opencue module."""

from __future__ import absolute_import
from __future__ import print_function
from __future__ import division

import logging

# pylint: disable=cyclic-import
from .cuebot import Cuebot
from . import api
from . import wrappers
from . import search

from .exception import CueException
from .exception import EntityNotFoundException
# pylint: disable=redefined-builtin
from .util import id
# pylint: enable=redefined-builtin
from .util import logPath
from .util import proxy
from .util import rep


class __NullHandler(logging.Handler):
    def emit(self, record):
        pass


__logger = logging.getLogger("opencue")
__logger.addHandler(__NullHandler())



#!/usr/bin/env python

#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for `opencue.wrappers.group`."""

from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest

import mock

from opencue_proto import job_pb2
import opencue.wrappers.group
import opencue.wrappers.job


TEST_GROUP_NAME = 'testGroup'


@mock.patch('opencue.cuebot.Cuebot.getStub')
class GroupTests(unittest.TestCase):

    def testCreateSubGroup(self, getStubMock):
        subgroupName = 'testSubgroup'
        stubMock = mock.Mock()
        stubMock.CreateSubGroup.return_value = job_pb2.GroupCreateSubGroupResponse(
            group=job_pb2.Group(name=subgroupName))
        getStubMock.return_value = stubMock

        group = opencue.wrappers.group.Group(
            job_pb2.Group(name=TEST_GROUP_NAME))
        subgroup = group.createSubGroup(subgroupName)

        stubMock.CreateSubGroup.assert_called_with(
            job_pb2.GroupCreateSubGroupRequest(group=group.data, name=subgroupName),
            timeout=mock.ANY)
        self.assertEqual(subgroup.name(), subgroupName)

    def testDelete(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Delete.return_value = job_pb2.GroupDeleteResponse()
        getStubMock.return_value = stubMock

        group = opencue.wrappers.group.Group(
            job_pb2.Group(name=TEST_GROUP_NAME))
        group.delete()

        stubMock.Delete.assert_called_with(
            job_pb2.GroupDeleteRequest(group=group.data),
            timeout=mock.ANY)

    def testSetName(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetName.return_value = job_pb2.GroupSetNameResponse()
        getStubMock.return_value = stubMock

        newName = 'changedName'
        group = opencue.wrappers.group.Group(
            job_pb2.Group(name=TEST_GROUP_NAME))
        group.setName(newName)

        stubMock.SetName.assert_called_with(
            job_pb2.GroupSetNameRequest(group=group.data, name=newName),
            timeout=mock.ANY)

    def testSetMaxCores(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetMaxCores.return_value = job_pb2.GroupSetMaxCoresResponse()
        getStubMock.return_value = stubMock

        value = 523
        group = opencue.wrappers.group.Group(
            job_pb2.Group(name=TEST_GROUP_NAME))
        group.setMaxCores(value)

        stubMock.SetMaxCores.assert_called_with(
            job_pb2.GroupSetMaxCoresRequest(group=group.data, max_cores=value),
            timeout=mock.ANY)

    def testSetMinCores(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetMinCores.return_value = job_pb2.GroupSetMinCoresResponse()
        getStubMock.return_value = stubMock

        value = 2
        group = opencue.wrappers.group.Group(
            job_pb2.Group(name=TEST_GROUP_NAME))
        group.setMinCores(value)

        stubMock.SetMinCores.assert_called_with(
            job_pb2.GroupSetMinCoresRequest(group=group.data, min_cores=value),
            timeout=mock.ANY)

    def testSetDefaultJobPriority(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetDefaultJobPriority.return_value = job_pb2.GroupSetDefJobPriorityResponse()
        getStubMock.return_value = stubMock

        value = 500
        group = opencue.wrappers.group.Group(
            job_pb2.Group(name=TEST_GROUP_NAME))
        group.setDefaultJobPriority(value)

        stubMock.SetDefaultJobPriority.assert_called_with(
            job_pb2.GroupSetDefJobPriorityRequest(group=group.data, priority=value),
            timeout=mock.ANY)

    def testSetDefaultJobMaxCores(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetDefaultJobMaxCores.return_value = job_pb2.GroupSetDefJobMaxCoresResponse()
        getStubMock.return_value = stubMock

        value = 523
        group = opencue.wrappers.group.Group(
            job_pb2.Group(name=TEST_GROUP_NAME))
        group.setDefaultJobMaxCores(value)

        stubMock.SetDefaultJobMaxCores.assert_called_with(
            job_pb2.GroupSetDefJobMaxCoresRequest(group=group.data, max_cores=value),
            timeout=mock.ANY)

    def testSetDefaultJobMinCores(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetDefaultJobMinCores.return_value = job_pb2.GroupSetDefJobMinCoresResponse()
        getStubMock.return_value = stubMock

        value = 2
        group = opencue.wrappers.group.Group(
            job_pb2.Group(name=TEST_GROUP_NAME))
        group.setDefaultJobMinCores(value)

        stubMock.SetDefaultJobMinCores.assert_called_with(
            job_pb2.GroupSetDefJobMinCoresRequest(group=group.data, min_cores=value),
            timeout=mock.ANY)

    def testGetGroups(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetGroups.return_value = job_pb2.GroupGetGroupsResponse(
            groups=job_pb2.GroupSeq(groups=[job_pb2.Group(name=TEST_GROUP_NAME)]))
        getStubMock.return_value = stubMock

        group = opencue.wrappers.group.Group(
            job_pb2.Group(name=TEST_GROUP_NAME))
        groups = group.getGroups()

        stubMock.GetGroups.assert_called_with(
            job_pb2.GroupGetGroupsRequest(group=group.data),
            timeout=mock.ANY)
        self.assertEqual(len(groups), 1)
        self.assertEqual(groups[0].name(), TEST_GROUP_NAME)

    def testGetJobs(self, getStubMock):
        jobName = 'testJob'
        stubMock = mock.Mock()
        stubMock.GetJobs.return_value = job_pb2.GroupGetJobsResponse(
            jobs=job_pb2.JobSeq(jobs=[job_pb2.Job(name=jobName)]))
        getStubMock.return_value = stubMock

        group = opencue.wrappers.group.Group(
            job_pb2.Group(name=TEST_GROUP_NAME))
        jobs = group.getJobs()

        stubMock.GetJobs.assert_called_with(
            job_pb2.GroupGetJobsRequest(group=group.data),
            timeout=mock.ANY)
        self.assertEqual(len(jobs), 1)
        self.assertEqual(jobs[0].name(), jobName)

    def testReparentJobs(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.ReparentJobs.return_value = job_pb2.GroupReparentJobsResponse()
        getStubMock.return_value = stubMock

        testJob = job_pb2.Job(name='testJob')
        testNestedJob = job_pb2.NestedJob(name='testNestedJob')
        jobs = [opencue.wrappers.job.Job(testJob), opencue.wrappers.job.NestedJob(testNestedJob)]
        group = opencue.wrappers.group.Group(
            job_pb2.Group(name=TEST_GROUP_NAME))
        group.reparentJobs(jobs)

        expected = job_pb2.JobSeq(jobs=[job_pb2.Job(name='testJob'),
                                        job_pb2.Job(name='testNestedJob',
                                                    job_stats=job_pb2.JobStats())])
        stubMock.ReparentJobs.assert_called_with(
            job_pb2.GroupReparentJobsRequest(group=group.data, jobs=expected),
            timeout=mock.ANY)

    def testReparentGroups(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.ReparentGroups.return_value = job_pb2.GroupReparentGroupsResponse()
        getStubMock.return_value = stubMock

        groups = [opencue.wrappers.group.Group(job_pb2.Group())]
        groupSeq = job_pb2.GroupSeq(groups=[grp.data for grp in groups])
        group = opencue.wrappers.group.Group(
            job_pb2.Group(name=TEST_GROUP_NAME))
        group.reparentGroups(groups)

        stubMock.ReparentGroups.assert_called_with(
            job_pb2.GroupReparentGroupsRequest(group=group.data, groups=groupSeq),
            timeout=mock.ANY)

    def testReparentGroupIds(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.ReparentGroups.return_value = job_pb2.GroupReparentGroupsResponse()
        getStubMock.return_value = stubMock

        groupId = 'ggg-gggg-ggg'
        groupIds = [groupId]
        groups = [opencue.wrappers.group.Group(job_pb2.Group(id='ggg-gggg-ggg'))]
        groupSeq = job_pb2.GroupSeq(groups=[grp.data for grp in groups])
        group = opencue.wrappers.group.Group(
            job_pb2.Group(name=TEST_GROUP_NAME))
        group.reparentGroupIds(groupIds)

        stubMock.ReparentGroups.assert_called_with(
            job_pb2.GroupReparentGroupsRequest(group=group.data, groups=groupSeq),
            timeout=mock.ANY)

    def testSetDepartment(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetDepartment.return_value = job_pb2.GroupSetDeptResponse()
        getStubMock.return_value = stubMock

        dept = 'pipeline'
        group = opencue.wrappers.group.Group(
            job_pb2.Group(name=TEST_GROUP_NAME))
        group.setDepartment(dept)

        expected = job_pb2.Group(name=TEST_GROUP_NAME)
        stubMock.SetDepartment.assert_called_with(
            job_pb2.GroupSetDeptRequest(group=expected, dept=dept),
            timeout=mock.ANY)

    def testSetGroup(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetGroup.return_value = job_pb2.GroupSetGroupResponse()
        getStubMock.return_value = stubMock

        parentGroup = opencue.wrappers.group.Group(job_pb2.Group(name='parentGroup'))
        group = opencue.wrappers.group.Group(
            job_pb2.Group(name=TEST_GROUP_NAME))
        group.setGroup(parentGroup)

        stubMock.SetGroup.assert_called_with(
            job_pb2.GroupSetGroupRequest(group=group.data, parent_group=parentGroup.data),
            timeout=mock.ANY)


@mock.patch('opencue.cuebot.Cuebot.getStub')
class NestedGroupTests(unittest.TestCase):

    def testAsGroup(self, getStubMock):
        nestedGroup = opencue.wrappers.group.NestedGroup(job_pb2.NestedGroup(
            id='ngn-ngng-ngn',
            name='testNestedGroup',
            department='pipeline'
        ))
        group = nestedGroup.asGroup()
        self.assertEqual(nestedGroup.data.id, group.data.id)
        self.assertEqual(nestedGroup.data.name, group.data.name)
        self.assertEqual(nestedGroup.data.department, group.data.department)



if __name__ == '__main__':
    unittest.main()



#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Custom exception classes for API error handling."""

from __future__ import print_function
from __future__ import division
from __future__ import absolute_import

import grpc

import opencue


class CueException(Exception):
    """A Base class for all client side cue exceptions"""
    failMsg = 'Caught an unknown server exception. Please check the server logs. {details}'
    retryMsg = 'Caught an unknown server exception, checking again...'
    retryable = False
    retryBackoff = 0.5  # seconds


class DeadlineExceededException(CueException):
    """Raised when the deadline for response has been exceeded."""
    failMsg = 'Request deadline exceeded. {details}'
    retryMsg = 'Request deadline exceeded, checking again...'


class EntityAlreadyExistsException(CueException):
    """Raised when the entity was not created because it already exists on the server"""
    failMsg = 'Object already exists. {details}'
    retryMsg = 'Object already exists, checking again...'


class EntityNotFoundException(CueException):
    """Raised when the entity was not found on the server."""
    failMsg = 'Object does not exist. {details}'
    retryMsg = 'Object does not exist, checking again...'


class CueInternalErrorException(CueException):
    """Raised when the server encountered a catchable error"""
    failMsg = 'Server caught an internal exception. {details}'
    retryMsg = 'Server caught an internal exception, checking again...'


class ConnectionException(CueException):
    """Raised when unable to connect to grpc server."""
    failMsg = 'Unable to contact grpc server. {details}'
    retryMsg = 'Unable to contact grpc server, checking again...'
    retryable = True


def getRetryCount():
    """Return the configured number of retries a cuebot call can make.
    If not specified in the config, all retryable calls will be called once and retried 3 times."""
    return opencue.cuebot.Cuebot.getConfig().get('cuebot.exception_retries', 3)


EXCEPTION_MAP = {
    grpc.StatusCode.NOT_FOUND: EntityNotFoundException,
    grpc.StatusCode.ALREADY_EXISTS: EntityAlreadyExistsException,
    grpc.StatusCode.DEADLINE_EXCEEDED: DeadlineExceededException,
    grpc.StatusCode.INTERNAL: CueInternalErrorException,
    grpc.StatusCode.UNAVAILABLE: ConnectionException
}



#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
# pylint: disable=missing-function-docstring,missing-module-docstring

import unittest

def create_test_suite():
    loader = unittest.TestLoader()
    start_dir = '.'  # Specify the directory where your test files reside
    suite = loader.discover(start_dir, pattern='test_*.py')
    return suite

if __name__ == '__main__':
    runner = unittest.TextTestRunner()
    test_suite = create_test_suite()
    runner.run(test_suite)



#!/usr/bin/env python

#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for `opencue.wrappers.frame`."""

from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import getpass
import os
import platform
import time
import unittest

import mock

from opencue_proto import depend_pb2
from opencue_proto import job_pb2
import opencue.wrappers.frame
import opencue.wrappers.job
import opencue.wrappers.layer


TEST_FRAME_NAME = 'testFrame'


@mock.patch('opencue.cuebot.Cuebot.getStub')
class FrameTests(unittest.TestCase):

    def testEat(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Eat.return_value = job_pb2.FrameEatResponse()
        getStubMock.return_value = stubMock

        frame = opencue.wrappers.frame.Frame(
            job_pb2.Frame(name=TEST_FRAME_NAME, state=job_pb2.WAITING))
        frame.eat()

        stubMock.Eat.assert_called_with(
            job_pb2.FrameEatRequest(frame=frame.data), timeout=mock.ANY)

    def testKill(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Kill.return_value = job_pb2.FrameKillResponse()
        getStubMock.return_value = stubMock

        frame = opencue.wrappers.frame.Frame(
            job_pb2.Frame(name=TEST_FRAME_NAME, state=job_pb2.RUNNING))
        username = getpass.getuser()
        pid = os.getpid()
        host_kill = platform.uname()[1]
        reason = "Frames Kill Request"
        frame.kill(username=username, pid=pid, host_kill=host_kill, reason=reason)

        stubMock.Kill.assert_called_with(
            job_pb2.FrameKillRequest(frame=frame.data,
                                     username=username,
                                     pid=str(pid),
                                     host_kill=host_kill,
                                     reason=reason), timeout=mock.ANY)

    def testRetry(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Retry.return_value = job_pb2.FrameRetryResponse()
        getStubMock.return_value = stubMock

        frame = opencue.wrappers.frame.Frame(
            job_pb2.Frame(name=TEST_FRAME_NAME, state=job_pb2.RUNNING))
        frame.retry()

        stubMock.Retry.assert_called_with(
            job_pb2.FrameRetryRequest(frame=frame.data), timeout=mock.ANY)

    def testGetWhatDependsOnThis(self, getStubMock):
        dependId = 'ddd-dddd-ddd'
        stubMock = mock.Mock()
        stubMock.GetWhatDependsOnThis.return_value = job_pb2.FrameGetWhatDependsOnThisResponse(
            depends=depend_pb2.DependSeq(depends=[depend_pb2.Depend(id=dependId)]))
        getStubMock.return_value = stubMock

        frame = opencue.wrappers.frame.Frame(
            job_pb2.Frame(name=TEST_FRAME_NAME, state=job_pb2.RUNNING))
        depends = frame.getWhatDependsOnThis()

        stubMock.GetWhatDependsOnThis.assert_called_with(
            job_pb2.FrameGetWhatDependsOnThisRequest(frame=frame.data), timeout=mock.ANY)
        self.assertEqual(len(depends), 1)
        self.assertEqual(depends[0].id(), dependId)

    def testGetWhatThisDependsOn(self, getStubMock):
        dependId = 'ddd-dddd-ddd'
        stubMock = mock.Mock()
        stubMock.GetWhatThisDependsOn.return_value = job_pb2.FrameGetWhatThisDependsOnResponse(
            depends=depend_pb2.DependSeq(depends=[depend_pb2.Depend(id=dependId)]))
        getStubMock.return_value = stubMock

        frame = opencue.wrappers.frame.Frame(
            job_pb2.Frame(name=TEST_FRAME_NAME, state=job_pb2.RUNNING))
        depends = frame.getWhatThisDependsOn()

        stubMock.GetWhatThisDependsOn.assert_called_with(
            job_pb2.FrameGetWhatThisDependsOnRequest(frame=frame.data), timeout=mock.ANY)
        self.assertEqual(len(depends), 1)
        self.assertEqual(depends[0].id(), dependId)

    def testCreateDependencyOnFrame(self, getStubMock):
        dependId = 'ddd-dddd-ddd'
        stubMock = mock.Mock()
        stubMock.CreateDependencyOnFrame.return_value = \
            job_pb2.FrameCreateDependencyOnFrameResponse(depend=depend_pb2.Depend(id=dependId))
        getStubMock.return_value = stubMock

        dependFrameName = 'frameDependTest'
        frame = opencue.wrappers.frame.Frame(
            job_pb2.Frame(name=TEST_FRAME_NAME))
        dependOnFrame = opencue.wrappers.frame.Frame(
            job_pb2.Frame(name=dependFrameName))
        depend = frame.createDependencyOnFrame(dependOnFrame)

        stubMock.CreateDependencyOnFrame.assert_called_with(
            job_pb2.FrameCreateDependencyOnFrameRequest(frame=frame.data,
                                                        depend_on_frame=dependOnFrame.data),
            timeout=mock.ANY)
        self.assertEqual(depend.id(), dependId)

    def testCreateDependencyOnJob(self, getStubMock):
        dependId = 'ddd-dddd-ddd'
        stubMock = mock.Mock()
        stubMock.CreateDependencyOnJob.return_value = \
            job_pb2.FrameCreateDependencyOnJobResponse(depend=depend_pb2.Depend(id=dependId))
        getStubMock.return_value = stubMock

        dependJobName = 'jobDependTest'
        frame = opencue.wrappers.frame.Frame(
            job_pb2.Frame(name=TEST_FRAME_NAME, state=job_pb2.RUNNING))
        dependOnJob = opencue.wrappers.job.Job(
            job_pb2.Job(name=dependJobName))
        depend = frame.createDependencyOnJob(dependOnJob)

        stubMock.CreateDependencyOnJob.assert_called_with(
            job_pb2.FrameCreateDependencyOnJobRequest(frame=frame.data, job=dependOnJob.data),
            timeout=mock.ANY)
        self.assertEqual(depend.id(), dependId)

    def testCreateDependencyOnLayer(self, getStubMock):
        dependId = 'ddd-dddd-ddd'
        stubMock = mock.Mock()
        stubMock.CreateDependencyOnLayer.return_value = \
            job_pb2.FrameCreateDependencyOnLayerResponse(depend=depend_pb2.Depend(id=dependId))
        getStubMock.return_value = stubMock

        dependLayerName = 'layerDependTest'
        frame = opencue.wrappers.frame.Frame(
            job_pb2.Frame(name=TEST_FRAME_NAME, state=job_pb2.RUNNING))
        dependOnLayer = opencue.wrappers.layer.Layer(
            job_pb2.Layer(name=dependLayerName))
        depend = frame.createDependencyOnLayer(dependOnLayer)

        stubMock.CreateDependencyOnLayer.assert_called_with(
            job_pb2.FrameCreateDependencyOnLayerRequest(frame=frame.data, layer=dependOnLayer.data),
            timeout=mock.ANY)
        self.assertEqual(depend.id(), dependId)

    def testMarkAsWaiting(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.MarkAsWaiting.return_value = job_pb2.FrameMarkAsWaitingResponse()
        getStubMock.return_value = stubMock

        frame = opencue.wrappers.frame.Frame(
            job_pb2.Frame(name=TEST_FRAME_NAME, state=job_pb2.RUNNING))
        frame.markAsWaiting()

        stubMock.MarkAsWaiting.assert_called_with(
            job_pb2.FrameMarkAsWaitingRequest(frame=frame.data), timeout=mock.ANY)

    def testDropDepends(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.DropDepends.return_value = job_pb2.FrameDropDependsResponse()
        getStubMock.return_value = stubMock

        target = depend_pb2.ANY_TARGET
        frame = opencue.wrappers.frame.Frame(job_pb2.Frame(name='arbitrary-frame-name'))
        frame.dropDepends(target)

        stubMock.DropDepends.assert_called_with(
            job_pb2.FrameDropDependsRequest(frame=frame.data, target=target),
            timeout=mock.ANY)

    def testRunTimeZero(self, getStubMock):
        zeroFrame = opencue.wrappers.frame.Frame(
            job_pb2.Frame(name=TEST_FRAME_NAME, start_time=0, stop_time=1000))
        self.assertEqual(zeroFrame.runTime(), 0)

    def testRunTimeRunning(self, getStubMock):
        curTime = int(time.time())
        startTime = 100
        expected = curTime - startTime
        runningFrame = opencue.wrappers.frame.Frame(
            job_pb2.Frame(name=TEST_FRAME_NAME, start_time=startTime, stop_time=0))
        threshold = abs(runningFrame.runTime() - expected)
        self.assertTrue(threshold < 1)

    def testRunTimeDone(self, getStubMock):
        startTime = 100
        stopTime = 500
        expected = stopTime - startTime
        runningFrame = opencue.wrappers.frame.Frame(
            job_pb2.Frame(name=TEST_FRAME_NAME, start_time=startTime, stop_time=stopTime))
        self.assertEqual(runningFrame.runTime(), expected)

    def testSetFrameStateDisplayOverride(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetFrameStateDisplayOverride.return_value = \
            job_pb2.FrameStateDisplayOverrideResponse()
        getStubMock.return_value = stubMock

        frame = opencue.wrappers.frame.Frame(job_pb2.Frame(name='TEST_FRAME_A'))
        frame.setFrameStateDisplayOverride(job_pb2.FrameState.SUCCEEDED,
                                           'TIMED_OUT',
                                           frame.STATUS_COLOR["BLUE"])

        stubMock.SetFrameStateDisplayOverride.assert_called_with(
            job_pb2.FrameStateDisplayOverrideRequest(frame=frame.data,
                override=job_pb2.FrameStateDisplayOverride(
                    state=job_pb2.FrameState.SUCCEEDED,
                    text='TIMED_OUT',
                    color=job_pb2.FrameStateDisplayOverride.RGB(red=0,
                                                                green=128,
                                                                blue=255))))

    def testGetFrameStateDisplayOverrides(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetFrameStateDisplayOverrides.return_value = \
            job_pb2.GetFrameStateDisplayOverridesResponse(
                overrides = job_pb2.FrameStateDisplayOverrideSeq(overrides=[
                    job_pb2.FrameStateDisplayOverride(state=job_pb2.FrameState.SUCCEEDED,
                                                      text='COMPLETE',
                                                      color=job_pb2.FrameStateDisplayOverride.RGB(
                                                          red=123, green=20, blue=102
                                                      )),
                    job_pb2.FrameStateDisplayOverride(state=job_pb2.FrameState.DEAD,
                                                      text='FAILED',
                                                      color=job_pb2.FrameStateDisplayOverride.RGB(
                                                          red=223, green=120, blue=10
                                                      ))]))
        getStubMock.return_value = stubMock

        frame = opencue.wrappers.frame.Frame(job_pb2.Frame(name='TEST_FRAME_A'))
        overrides = frame.getFrameStateDisplayOverrides()

        stubMock.GetFrameStateDisplayOverrides.assert_called_with(
            job_pb2.GetFrameStateDisplayOverridesRequest(frame=frame.data))

        self.assertTrue(len(overrides), 2)
        self.assertEqual(overrides[0].state, job_pb2.FrameState.SUCCEEDED)
        self.assertEqual(overrides[1].state, job_pb2.FrameState.DEAD)


class FrameEnumTests(unittest.TestCase):

    def testCheckpointState(self):
        self.assertEqual(opencue.api.Frame.CheckpointState.DISABLED,
                         job_pb2.DISABLED)
        self.assertEqual(opencue.api.Frame.CheckpointState.DISABLED, 0)

    def testFrameExitStatus(self):
        self.assertEqual(opencue.api.Frame.FrameExitStatus.FAILED_LAUNCH,
                         job_pb2.FAILED_LAUNCH)
        self.assertEqual(opencue.api.Frame.FrameExitStatus.FAILED_LAUNCH, 256)

    def testFrameState(self):
        self.assertEqual(opencue.api.Frame.FrameState.RUNNING,
                         job_pb2.RUNNING)
        self.assertEqual(opencue.api.Frame.FrameState.RUNNING, 2)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for `opencue.config`."""

import os
import unittest

import mock
import pyfakefs.fake_filesystem_unittest

import opencue.config


EXPECTED_DEFAULT_CONFIG = {
    'logger.format': '%(levelname)-9s %(module)-10s %(message)s',
    'logger.level': 'WARNING',
    'cuebot.protocol': 'tcp',
    'cuebot.grpc_port': 8443,
    'cuebot.timeout': 10000,
    'cuebot.max_message_bytes': 104857600,
    'cuebot.exception_retries': 3,
    'cuebot.facility_default': 'local',
    'cuebot.facility': {
        'local': ['localhost:8443'],
        'dev': ['cuetest02-vm.example.com:8443'],
        'cloud': [
            'cuebot1.example.com:8443',
            'cuebot2.example.com:8443',
            'cuebot3.example.com:8443'
        ],
    },
}

USER_CONFIG = """
cuebot.facility_default: fake-facility-01
cuebot.facility:
  fake-facility-01:
    - fake-cuebot-01:1234
  fake-facility-02:
    - fake-cuebot-02:5678
    - fake-cuebot-03:9012
"""


class ConfigTests(pyfakefs.fake_filesystem_unittest.TestCase):
    def setUp(self):
        self.setUpPyfakefs()
        self.fs.add_real_file(
            os.path.join(os.path.dirname(opencue.__file__), 'default.yaml'), read_only=True)
        if 'OPENCUE_CONFIG_FILE' in os.environ:
            del os.environ['OPENCUE_CONFIG_FILE']
        if 'OPENCUE_CONF' in os.environ:
            del os.environ['OPENCUE_CONF']

    @mock.patch('platform.system', new=mock.Mock(return_value='Linux'))
    @mock.patch('os.path.expanduser', new=mock.Mock(return_value='/home/username'))
    def test__should_return_config_dir_unix(self):
        self.assertEqual('/home/username/.config/opencue', opencue.config.config_base_directory())

    @mock.patch('platform.system', new=mock.Mock(return_value='Windows'))
    @mock.patch(
        'os.path.expandvars', new=mock.Mock(return_value='C:/Users/username/AppData/Roaming'))
    def test__should_return_config_dir_windows(self):
        self.assertEqual(
            'C:/Users/username/AppData/Roaming/opencue', opencue.config.config_base_directory())

    def test__should_load_default_config(self):
        self.assertIsNone(os.environ.get('OPENCUE_CONFIG_FILE'))
        self.assertIsNone(os.environ.get('OPENCUE_CONF'))

        config = opencue.config.load_config_from_file()

        self.assertEqual(EXPECTED_DEFAULT_CONFIG, config)

    def test__should_load_user_config(self):
        config_file_path = '/path/to/config.yaml'
        self.fs.create_file(config_file_path, contents=USER_CONFIG)
        os.environ['OPENCUE_CONFIG_FILE'] = config_file_path
        # Define some invalid config using the old setting name, this ensures the old env var
        # will be ignored if the new one is set.
        config_file_path_legacy = '/path/to/legacy/config.yaml'
        self.fs.create_file(config_file_path_legacy, contents='invalid yaml')
        os.environ['OPENCUE_CONF'] = config_file_path_legacy

        config = opencue.config.load_config_from_file()

        self.assertEqual('fake-facility-01', config['cuebot.facility_default'])
        self.assertEqual(['fake-cuebot-01:1234'], config['cuebot.facility']['fake-facility-01'])
        self.assertEqual(
            ['fake-cuebot-02:5678', 'fake-cuebot-03:9012'],
            config['cuebot.facility']['fake-facility-02'])
        # Settings not defined in user config should still have default values.
        self.assertEqual(10000, config['cuebot.timeout'])
        self.assertEqual(3, config['cuebot.exception_retries'])

    def test__should_load_user_config_from_legacy_var(self):
        config_file_path = '/path/to/legacy/config.yaml'
        self.fs.create_file(config_file_path, contents=USER_CONFIG)
        os.environ['OPENCUE_CONF'] = config_file_path

        config = opencue.config.load_config_from_file()

        self.assertEqual('fake-facility-01', config['cuebot.facility_default'])
        self.assertEqual(['fake-cuebot-01:1234'], config['cuebot.facility']['fake-facility-01'])
        self.assertEqual(
            ['fake-cuebot-02:5678', 'fake-cuebot-03:9012'],
            config['cuebot.facility']['fake-facility-02'])
        # Settings not defined in user config should still have default values.
        self.assertEqual(10000, config['cuebot.timeout'])
        self.assertEqual(3, config['cuebot.exception_retries'])

    @mock.patch('platform.system', new=mock.Mock(return_value='Linux'))
    @mock.patch('os.path.expanduser', new=mock.Mock(return_value='/home/username'))
    def test__should_load_user_config_from_user_profile(self):
        config_file_path = '/home/username/.config/opencue/opencue.yaml'
        self.fs.create_file(config_file_path, contents=USER_CONFIG)

        config = opencue.config.load_config_from_file()

        self.assertEqual('fake-facility-01', config['cuebot.facility_default'])
        self.assertEqual(['fake-cuebot-01:1234'], config['cuebot.facility']['fake-facility-01'])
        self.assertEqual(
            ['fake-cuebot-02:5678', 'fake-cuebot-03:9012'],
            config['cuebot.facility']['fake-facility-02'])
        # Settings not defined in user config should still have default values.
        self.assertEqual(10000, config['cuebot.timeout'])
        self.assertEqual(3, config['cuebot.exception_retries'])


if __name__ == '__main__':
    unittest.main()



#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Module for classes related to procs."""

import enum

from opencue_proto import host_pb2
from opencue.cuebot import Cuebot
import opencue.wrappers.frame
import opencue.wrappers.host
import opencue.wrappers.job
import opencue.wrappers.layer


class Proc(object):
    """This class contains the grpc implementation related to a Proc.

    A proc is a bookable unit of a host. Hosts may contain many procs; each proc can be assigned
    a different frame to work on."""

    class RedirectType(enum.IntEnum):
        """Represents the type of a proc redirect."""
        JOB_REDIRECT = host_pb2.JOB_REDIRECT
        GROUP_REDIRECT = host_pb2.GROUP_REDIRECT

    class RunState(enum.IntEnum):
        """Represents the current state of a proc."""
        IDLE = host_pb2.IDLE
        BOOKED = host_pb2.BOOKED

    def __init__(self, proc=None):
        self.data = proc
        self.stub = Cuebot.getStub('proc')

    def kill(self):
        """Kills the frame running on this proc."""
        response = self.stub.Kill(host_pb2.ProcKillRequest(proc=self.data), timeout=Cuebot.Timeout)
        return response

    def unbook(self, kill=False):
        """Unbooks the current frame from this proc.

        :type kill: bool
        :param kill: if true, the frame will be immediately killed
        """
        response = self.stub.Unbook(
            host_pb2.ProcUnbookRequest(proc=self.data, kill=kill), timeout=Cuebot.Timeout)
        return response

    def redirectToJob(self, job, kill=False):
        """Unbooks the current frame from this proc and redirects the proc to a specific job.

        :type job: opencue.wrappers.job.Job
        :param job: job which the proc should be booked to
        :type kill: bool
        :param kill: if true, the frame will be immediately killed
        """
        self.stub.RedirectToJob(
            host_pb2.ProcRedirectToJobRequest(proc=self.data, job_id=job.data.id, kill=kill),
            timeout=Cuebot.Timeout)

    def redirectToGroup(self, group, kill=False):
        """Unbooks the current frame from this proc and redirects the proc to another group.

        :type group: opencue.wrappers.group.Group
        :param group: group which the proc should be booked to
        :type kill: bool
        :param kill: if true, the frame will be immediately killed
        """
        self.stub.RedirectToGroup(
            host_pb2.ProcRedirectToGroupRequest(proc=self.data, group_id=group.data.id, kill=kill),
            timeout=Cuebot.Timeout)

    def getHost(self):
        """Returns the host this proc is allocated from.

        :rtype:  opencue.wrappers.host.Host
        :return: the host this proc is allocated from
        """
        response = self.stub.GetHost(host_pb2.ProcGetHostRequest(proc=self.data),
                                     timeout=Cuebot.Timeout)
        return opencue.wrappers.host.Host(response.host)

    def getFrame(self):
        """Returns the frame this proc is running.

        :rtype:  opencue.wrappers.frame.Frame
        :return: the frame this proc is running
        """
        response = self.stub.GetFrame(host_pb2.ProcGetFrameRequest(proc=self.data),
                                      timeout=Cuebot.Timeout)
        return opencue.wrappers.frame.Frame(response.frame)

    def getLayer(self):
        """Returns the layer this proc is running.

        :rtype:  opencue.wrappers.layer.Layer
        :return: the layer this proc is running
        """
        response = self.stub.GetLayer(host_pb2.ProcGetLayerRequest(proc=self.data),
                                      timeout=Cuebot.Timeout)
        return opencue.wrappers.layer.Layer(response.layer)

    def getJob(self):
        """Returns the job this proc is running.

        :rtype:  opencue.wrappers.job.Job
        :return: the job this proc is running
        """
        response = self.stub.GetJob(host_pb2.ProcGetJobRequest(proc=self.data),
                                    timeout=Cuebot.Timeout)
        return opencue.wrappers.job.Job(response.job)

    def id(self):
        """Returns the id of the proc.

        :rtype:  str
        :return: id of the proc
        """
        return self.data.id

    def name(self):
        """Returns the name of the proc.

        :rtype:  str
        :return: name of the proc
        """
        return self.data.name

    def jobName(self):
        """Returns the name of the job of the frame running on the proc.

        :rtype:  str
        :return: name of the current job"""
        return self.data.job_name

    def frameName(self):
        """Returns the name of the frame on the proc.

        :rtype:  str
        :return: name of the current frame
        """
        return self.data.frame_name

    def showName(self):
        """Returns the name of the show of the frame running on the proc.

        :rtype:  str
        :return: name of the current show
        """
        return self.data.show_name

    def coresReserved(self):
        """The number of cores reserved for this frame.

        :rtype:  float
        :return: cores reserved for the running frame
        """
        return self.data.reserved_cores

    def memReserved(self):
        """The amount of memory reserved for the running frame.

        :rtype:  int
        :return: memory reserved for the running frame in kB
        """
        return self.data.reserved_memory

    def memUsed(self):
        """The amount of memory used by the running frame.

        :rtype:  int
        :return: memory used by the running frame in kB
        """
        return self.data.used_memory

    def bookedTime(self):
        """The last time this proc was booked to a job.

        :rtype: int
        :return: last time booked as an epoch
        """
        return self.data.booked_time

    def dispatchTime(self):
        """The last time this proc was dispatched work.

        :rtype: int
        :return: last time dispatched as an epoch
        """
        return self.data.dispatch_time

    def isUnbooked(self):
        """Returns whether this proc is unbooked.

        :rtype: bool
        :return: whether the proc is unbooked
        """
        return self.data.unbooked


class NestedProc(Proc):
    """This class contains information and actions related to a nested job."""

    def __init__(self, nestedProc):
        super(NestedProc, self).__init__(nestedProc)
        # job children are most likely empty but its possible to
        # populate this with NestedLayer objects.
        self.__children = []

    def children(self):
        """Returns children of the proc."""
        return self.__children



#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Module for classes related to frames."""

import enum
import getpass
import time
import os
import platform

from opencue_proto import job_pb2
from opencue import Cuebot
import opencue.wrappers.depend


class Frame(object):
    """This class contains the grpc implementation related to a Frame."""

    class CheckpointState(enum.IntEnum):
        """Possible states for a frame's checkpointing status, if it uses that."""
        DISABLED = job_pb2.DISABLED
        ENABLED = job_pb2.ENABLED
        COPYING = job_pb2.COPYING
        COMPLETE = job_pb2.COMPLETE

    class FrameExitStatus(enum.IntEnum):
        """Possible frame exit statuses."""
        SUCCESS = job_pb2.SUCCESS
        FAILED_LAUNCH = job_pb2.FAILED_LAUNCH
        SKIP_RETRY = job_pb2.SKIP_RETRY

    class FrameState(enum.IntEnum):
        """Possible frame states."""
        WAITING = job_pb2.WAITING
        SETUP = job_pb2.SETUP
        RUNNING = job_pb2.RUNNING
        SUCCEEDED = job_pb2.SUCCEEDED
        DEPEND = job_pb2.DEPEND
        DEAD = job_pb2.DEAD
        EATEN = job_pb2.EATEN
        CHECKPOINT = job_pb2.CHECKPOINT

    STATUS_COLOR = {
        "RED": (255, 0, 0),
        "GREEN": (0, 255, 0),
        "BLUE": (0, 128, 255),
        "YELLOW": (255, 255, 0),
        "ORANGE": (255, 128, 0),
        "PURPLE": (127, 0, 255),
        "PINK": (255, 51, 255)
    }


    def __init__(self, frame=None):
        self.data = frame
        self.stub = Cuebot.getStub('frame')

    def eat(self):
        """Eats the frame."""
        if self.data.state != job_pb2.FrameState.Value('EATEN'):
            self.stub.Eat(job_pb2.FrameEatRequest(frame=self.data), timeout=Cuebot.Timeout)

    def kill(self, username=None, pid=None, host_kill=None, reason=None):
        """Kills the frame."""
        username = username if username else getpass.getuser()
        pid = pid if pid else os.getpid()
        host_kill = host_kill if host_kill else platform.uname()[1]
        if self.data.state == job_pb2.FrameState.Value('RUNNING'):
            self.stub.Kill(job_pb2.FrameKillRequest(frame=self.data,
                                                    username=username,
                                                    pid=str(pid),
                                                    host_kill=host_kill,
                                                    reason=reason),
                           timeout=Cuebot.Timeout)

    def retry(self):
        """Retries the frame."""
        if self.data.state != job_pb2.FrameState.Value('WAITING'):
            self.stub.Retry(job_pb2.FrameRetryRequest(frame=self.data), timeout=Cuebot.Timeout)

    def addRenderPartition(self, hostname, threads, max_cores, max_mem, max_gpu_memory, max_gpus):
        """Adds a render partition to the frame.

        :type  hostname: str
        :param hostname: hostname of the partition
        :type  threads: int
        :param threads: number of threads of the partition
        :type  max_cores: int
        :param max_cores: max cores enabled for the partition
        :type  max_mem: int
        :param max_mem: amount of memory reserved for the partition
        :type  max_gpu_memory: int
        :param max_gpu_memory: max gpu memory enabled for the partition
        :type  max_gpus: int
        :param max_gpus: max number of gpus enabled for the partition
        """
        self.stub.AddRenderPartition(
            job_pb2.FrameAddRenderPartitionRequest(
                frame=self.data,
                host=hostname,
                threads=threads,
                max_cores=max_cores,
                max_memory=max_mem,
                max_gpu_memory=max_gpu_memory,
                username=os.getenv("USER", "unknown"),
                max_gpu=max_gpus))

    def getWhatDependsOnThis(self):
        """Returns a list of dependencies that depend directly on this frame.

        :rtype:  list<opencue.wrappers.depend.Depend>
        :return: list of dependencies that depend directly on this frame
        """
        response = self.stub.GetWhatDependsOnThis(
            job_pb2.FrameGetWhatDependsOnThisRequest(frame=self.data),
            timeout=Cuebot.Timeout)
        return [opencue.wrappers.depend.Depend(dep) for dep in response.depends.depends]

    def getWhatThisDependsOn(self):
        """Returns a list of dependencies that this frame depends on.

        :rtype:  list<opencue.wrappers.depend.Depend>
        :return: list of dependencies that this frame depends on
        """
        response = self.stub.GetWhatThisDependsOn(
            job_pb2.FrameGetWhatThisDependsOnRequest(frame=self.data),
            timeout=Cuebot.Timeout)
        return [opencue.wrappers.depend.Depend(dep) for dep in response.depends.depends]

    def createDependencyOnJob(self, job):
        """Creates and returns a frame-on-job dependency.

        :type  job: opencue.wrappers.job.Job
        :param job: the job you want this frame to depend on
        :rtype:  opencue.wrappers.depend.Depend
        :return: The new dependency
        """
        response = self.stub.CreateDependencyOnJob(
            job_pb2.FrameCreateDependencyOnJobRequest(frame=self.data, job=job.data),
            timeout=Cuebot.Timeout)
        return opencue.wrappers.depend.Depend(response.depend)

    def createDependencyOnLayer(self, layer):
        """Creates and returns a frame-on-layer dependency.

        :type  layer: opencue.wrappers.layer.Layer
        :param layer: the layer you want this frame to depend on
        :rtype:  opencue.wrappers.depend.Depend
        :return: the new dependency
        """
        response = self.stub.CreateDependencyOnLayer(
            job_pb2.FrameCreateDependencyOnLayerRequest(frame=self.data, layer=layer.data),
            timeout=Cuebot.Timeout)
        return opencue.wrappers.depend.Depend(response.depend)

    def createDependencyOnFrame(self, frame):
        """Creates and returns a frame-on-frame dependency.

        :type  frame: opencue.wrappers.frame.Frame
        :param frame: the frame you want this frame to depend on
        :rtype:  opencue.wrappers.depend.Depend
        :return: the new dependency
        """
        frame_dep = frame.data if isinstance(frame, type(self)) else frame
        response = self.stub.CreateDependencyOnFrame(
            job_pb2.FrameCreateDependencyOnFrameRequest(frame=self.data,
                                                        depend_on_frame=frame_dep),
                                                        timeout=Cuebot.Timeout)
        return opencue.wrappers.depend.Depend(response.depend)

    def dropDepends(self, target):
        """Drops every dependency that is causing this frame not to run."""
        self.stub.DropDepends(
            job_pb2.FrameDropDependsRequest(frame=self.data, target=target),
            timeout=Cuebot.Timeout)

    def markAsWaiting(self):
        """Marks the frame as waiting; ready to run.

        Similar to dropDepends. The frame will be able to run even if the job has an external
        dependency."""
        self.stub.MarkAsWaiting(
            job_pb2.FrameMarkAsWaitingRequest(frame=self.data),
            timeout=Cuebot.Timeout)

    def setCheckpointState(self, checkPointState):
        """Sets the checkPointState of the frame.

        :type  checkPointState: job_pb.CheckpointState
        :param checkPointState: the checkpoint state of the frame
        """
        self.stub.SetCheckpointState(
            job_pb2.FrameSetCheckpointStateRequest(frame=self.data, state=checkPointState))

    def setFrameStateDisplayOverride(self, status, override_text, override_rgb):
        """
        Override the displayed text of a frame status.
        If an override already exists for the frame-state combo, the existing
        override will be updated to the new text and color values.
        If the override is identical to an existing override, no-op.

        :param status: the job_pb2.FrameState to override
        :param override_text: the text to display
        :param override_rgb: tuple containing the RGB int values e.g.(255, 0, 0)
        :return:
        """
        override = job_pb2.FrameStateDisplayOverride(state=status,
                                text=override_text,
                                color=job_pb2.FrameStateDisplayOverride.RGB(
                                    red=override_rgb[0],
                                    green=override_rgb[1],
                                    blue=override_rgb[2]))
        self.stub.SetFrameStateDisplayOverride(
            job_pb2.FrameStateDisplayOverrideRequest(frame=self.data,
                                                     override=override))

    def getFrameStateDisplayOverrides(self):
        """
        Retrieve all frame state display overrides for the frame

        :rtype: list
        :return: overrides for the frame
        """
        response = self.stub.GetFrameStateDisplayOverrides(
            job_pb2.GetFrameStateDisplayOverridesRequest(frame=self.data))
        return response.overrides.overrides

    def hasFrameStateDisplayOverride(self):
        """
        Check if frame has state override

        :rtype: boolean
        :return:  if a frame has any state overrides or not
        """
        if self.data.HasField("frame_state_display_override"):
            return True
        return False

    def id(self):
        """Returns the id of the frame.

        :rtype:  str
        :return: id of the frame
        """
        return self.data.id

    def name(self):
        """Returns the name of the frame.

        :rtype:  str
        :return: name of the frame
        """
        return "%04d-%s" % (self.data.number, self.data.layer_name)

    def layer(self):
        """Returns the name of the layer name that the frame belongs to.

        :rtype:  str
        :return: name of the layer
        """
        return self.data.layer_name

    def frame(self):
        """Returns the frame number as a padded string.

        :rtype:  str
        :return: frame number padded to 4 digits
        """
        return "%04d" % self.data.number

    def number(self):
        """Returns the frame number, unpadded.

        :rtype:  int
        :return: frame number
        """
        return self.data.number

    def dispatchOrder(self):
        """Returns the frame's dispatch order.

        :rtype:  int
        :return: frame dispatch order
        """
        return self.data.dispatch_order

    def startTime(self):
        """Returns the epoch timestamp of the frame's start time.

        :rtype:  int
        :return: frame start time as an epoch
        """
        return self.data.start_time

    def stopTime(self):
        """Returns the epoch timestamp of the frame's stop time.

        :rtype:  int
        :return: frame stop time as an epoch
        """
        return self.data.stop_time

    def resource(self):
        """Returns the most recent resource that the frame has started running on.

        Ex: vrack999/1.0 = host/proc:cores

        :rtype:  str
        :return: most recent running resource
        """
        return self.data.last_resource

    def retries(self):
        """Returns the number of times the frame has been retried.

        :rtype:  int
        :return: number of retries
        """
        return self.data.retry_count

    def exitStatus(self):
        """Returns the frame's exit status.

        :rtype:  int
        :return: last exit status of the frame
        """
        return self.data.exit_status

    def maxRss(self):
        """Returns the frame's maxRss.

        :rtype:  long
        :return: max RSS in Kb
        """
        return self.data.max_rss

    def memUsed(self):
        """Returns the frame's currently used memory.

        :rtype:  long
        :return: currently used memory in Kb
        """
        return self.data.used_memory

    def memReserved(self):
        """Returns the frame's currently reserved memory.

        :rtype:  long
        :return: currently reserved memory in Kb
        """
        return self.data.reserved_memory

    def state(self): # call it status?
        """Returns the state of the frame.

        :rtype:  job_pb2.FrameState
        :return: state of the frame
        """
        return self.data.state

    def runTime(self):
        """Returns the number of seconds that the frame has been (or was) running.

        :rtype:  int
        :return: frame runtime in seconds
        """
        if self.data.start_time == 0:
            return 0
        if self.data.stop_time == 0:
            return int(time.time() - self.data.start_time)
        return self.data.stop_time - self.data.start_time

    def frameStateDisplayOverride(self):
        """ Returns the frame state display override if there is one.
            Meant to be used in conjunction with "hasFrameStateDisplayOverride"

            :rtype: job_pb2.FrameStateDisplayOverride
            :return: frame state display override or None
        """
        if self.hasFrameStateDisplayOverride():
            return self.data.frame_state_display_override
        return None



#!/usr/bin/env python

#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for `opencue.cuebot`."""

import os
import unittest
import mock

import opencue


TESTING_CONFIG = {
    "cuebot.facility_default": "fake-facility-01",
    "cuebot.facility": {
        "fake-facility-01": [
            "fake-cuebot-01",
        ],
        "fake-facility-02": [
            "fake-cuebot-02",
            "fake-cuebot-03",
        ],
    },
}


class CuebotTests(unittest.TestCase):
    def setUp(self):
        self.cuebot = opencue.Cuebot()
        # Mocking the cue service ensures the initial healthcheck request made to Cuebot
        # will succeed.
        self.cuebot.SERVICE_MAP['cue'] = mock.Mock()

    def test__should_set_hosts_and_channel(self):
        healthcheck_mock = mock.Mock()
        self.cuebot.SERVICE_MAP['cue'] = healthcheck_mock

        # Clear any existing overrides
        if 'CUEBOT_HOSTS' in os.environ:
            del os.environ['CUEBOT_HOSTS']
        self.cuebot.init(config=TESTING_CONFIG)

        self.assertEqual(["fake-cuebot-01"], self.cuebot.Hosts)
        self.assertIsNotNone(self.cuebot.RpcChannel)
        healthcheck_mock.assert_called_with(self.cuebot.RpcChannel)

    def test__should_set_known_facility(self):
        self.cuebot.init(config=TESTING_CONFIG)

        self.cuebot.setHostWithFacility('fake-facility-02')

        self.assertEqual(['fake-cuebot-02', 'fake-cuebot-03'], self.cuebot.Hosts)

    def test__should_ignore_unknown_facility(self):
        self.cuebot.init(config=TESTING_CONFIG)

        self.cuebot.setHostWithFacility('unknown-facility')

        self.assertEqual(['fake-cuebot-01'], self.cuebot.Hosts)


if __name__ == '__main__':
    unittest.main()



#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Module for classes related to subscriptions."""

from opencue_proto import subscription_pb2
from opencue.cuebot import Cuebot


class Subscription(object):
    """This class contains the grpc implementation related to a Subscription.

    A subscription associates a show with an allocation. It allows the show to utilize resources
    within that allocation and determines the amount of resources the show is allowed to use at
    any given time."""

    def __init__(self, subscription=None):
        self.data = subscription
        self.stub = Cuebot.getStub('subscription')

    def find(self, name):
        """Returns a subscription by name.

        :type  name: str
        :param name: name of subscription to find
        :rtype:  opencue.wrappers.subscription.Subscription
        :return: the named subscription
        """
        response = self.stub.Find(
            subscription_pb2.SubscriptionFindRequest(name=name),
            timeout=Cuebot.Timeout)
        return Subscription(response.subscription)

    def get(self, subscription_id):
        """Returns a subscription by id.

        :type  subscription_id: str
        :param subscription_id: id of subscription to get
        :rtype:  opencue.wrappers.subscription.Subscription
        :return: the subscription of the id
        """
        response = self.stub.Get(
            subscription_pb2.SubscriptionGetRequest(id=subscription_id),
            timeout=Cuebot.Timeout)
        return Subscription(response.subscription)

    def setSize(self, size):
        """Sets subscription size; the number of cores the show is allowed to use consistently.

        :type  size: int
        :param size: the new subscription size
        """
        assert (isinstance(size, int)), "size is not expected type: int"
        self.stub.SetSize(
            subscription_pb2.SubscriptionSetSizeRequest(subscription=self.data, new_size=size),
            timeout=Cuebot.Timeout)

    def setBurst(self, burst):
        """Sets subscription burst size; the number of cores the show is allowed to burst to.

        :type  burst: int
        :param burst: the new burst size
        """
        assert (isinstance(burst, int)), "burst is not expected type: int"
        self.stub.SetBurst(
            subscription_pb2.SubscriptionSetBurstRequest(subscription=self.data, burst=burst),
            timeout=Cuebot.Timeout)

    def delete(self):
        """Deletes a subscription."""
        self.stub.Delete(
            subscription_pb2.SubscriptionDeleteRequest(subscription=self.data),
            timeout=Cuebot.Timeout)

    def id(self):
        """Returns the id of the subscription.

        :rtype:  str
        :return: id of the subscription
        """
        return self.data.id

    def name(self):
        """Returns the name of the subscription.

        Subscription names follow the format `<allocation name>.<show name>`, which can also be
        read as `<facility name>.<tag>.<show name>` due to the way allocations are named. For
        example the subscription name `local.general.testing` would indicate the show `testing`
        has a subscription to the `local.general` allocation. The `local.general` allocation
        indicates all hosts in the facility `local` containing the `general` tag.

        :rtype:  str
        :return: name of the subscription
        """
        return self.data.name

    def size(self):
        """Returns the subscription size.

        :rtype:  int
        :return: subscription size
        """
        return self.data.size

    def burst(self):
        """Returns the subscription burst size.

        :rtype:  int
        :return: subscription burst size
        """
        return self.data.burst

    def reservedCores(self):
        """Returns the current number of cores reserved in this subscription.

        :rtype:  float
        :return: reserved cores
        """
        return self.data.reserved_cores

    def show(self):
        """Returns the name of the show that this subscription is for.

        :rtype:  str
        :return: the name of the show that this subscription is for
        """
        return self.data.show_name

    def allocation(self):
        """Returns the allocation that this subscription is for.

        :rtype:  str
        :return: the allocation subscribed to
        """
        return self.data.allocation_name



#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""
Helper class for representing a frame seqeunce path.

It supports a complex syntax implementing features such as comma-separated frame ranges,
stepped frame ranges and more. See the FileSequence class for more detail.
"""

import re
from .FrameSet import FrameSet


class FileSequence:
    """Represents a file path to a frame sequence"""
    __filepath = None
    frameSet = None
    __basename = None
    __prefix = None
    __suffix = None
    __dirname = ""
    __padSize = 1
    __iter_index = 0

    def __init__(self, filepath):
        """
        Construct a FileSequence object by parsing a filepath.
        Details on how to specify a frame range can be seen in the FrameRange class
        """

        filePathMatch = re.match(r"^(?P<pf>.*\.)(?P<fspec>[\d#,\-@x]+)(?P<sf>\.[.\w]+)$", filepath)
        if filePathMatch is not None:
            self.__filepath = filepath
            self.__prefix = filePathMatch.group('pf')
            self.__suffix = filePathMatch.group('sf')
            dirmatch = re.match(r"^(?P<dirname>.*/)(?P<basename>.*)$", self.__prefix)
            if dirmatch is not None:
                self.__dirname = dirmatch.group("dirname")
                self.__basename = dirmatch.group("basename")
            else:
                self.__basename = self.__prefix
            framerangematch = re.match(r"^([\d\-x,]+)", filePathMatch.group("fspec"))
            if framerangematch is not None:
                self.frameSet = FrameSet(framerangematch.group(1))
                if self.frameSet.get(0) > self.frameSet.get(-1):
                    raise ValueError('invalid filesequence range : ' + framerangematch.group(1))
                firstFrameMatch = re.findall(r"^[-0]\d+", framerangematch.group(1))
                if len(firstFrameMatch) > 0:
                    self.__padSize = len(firstFrameMatch[0])

            padmatch = re.findall(r"#+$", filePathMatch.group("fspec"))
            if len(padmatch) > 0:
                self.__padSize = len(padmatch[0])
        else:
            raise ValueError('invalid filesequence path : ' + filepath)

    def getPrefix(self):
        """Returns the prefix of the file sequence"""
        return self.__prefix

    def getSuffix(self):
        """Returns the suffix of the file sequence"""
        return self.__suffix

    def getDirname(self):
        """Returns the dirname of the file sequence, if given otherwise returns empty an string"""
        return self.__dirname

    def getBasename(self):
        """Returns the base name of the file sequence"""
        return self.__basename.rstrip(".")

    def getPadSize(self):
        """Returns the size of the frame padding. It defaults to 1 if none is detected"""
        return self.__padSize

    def getFileList(self, frameSet=None):
        """ Returns the file list of the sequence """
        filelist = []
        paddingString = "%%0%dd" % self.getPadSize()
        if self.frameSet:
            for frame in self.frameSet.getAll():
                if (frameSet is None or
                        (isinstance(frameSet, FrameSet) and frame in frameSet.getAll())):
                    framepath = self.getPrefix() + paddingString % frame + self.getSuffix()
                    filelist.append(framepath)
        else:
            for frame in frameSet.getAll():
                framepath = self.getPrefix() + paddingString % frame + self.getSuffix()
                filelist.append(framepath)
        return filelist

    def getOpenRVPath(self, frameSet=None):
        """ Returns a string specific for the OpenRV player"""
        frameRange = ""
        curFrameSet = frameSet or self.frameSet
        if isinstance(curFrameSet, FrameSet):
            frameRange = "%d-%d" % (curFrameSet.get(0), curFrameSet.get(-1))
        framepath = self.getPrefix() + frameRange + "@"*self.__padSize + self.getSuffix()
        return framepath

    def __getitem__(self, index):
        return self.getFileList()[index]

    def __next__(self):
        self.__iter_index += 1
        if self.__iter_index <= len(self):
            return self.getFileList()[self.__iter_index - 1]
        raise StopIteration

    def __iter__(self):
        self.__iter_index = 0
        return self

    def __len__(self):
        return len(self.getFileList())

    def __call__(self, frame):
        paddingString = "%%0%dd" % self.getPadSize()
        framepath = self.getPrefix() + paddingString % frame + self.getSuffix()
        return framepath

    def __str__(self):
        return self.__filepath



#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""The OpenCue static API."""

from __future__ import absolute_import
from __future__ import print_function
from __future__ import division

from opencue_proto import comment_pb2
from opencue_proto import criterion_pb2
from opencue_proto import cue_pb2
from opencue_proto import department_pb2
from opencue_proto import depend_pb2
from opencue_proto import facility_pb2
from opencue_proto import filter_pb2
from opencue_proto import host_pb2
from opencue_proto import job_pb2
from opencue_proto import limit_pb2
from opencue_proto import renderPartition_pb2
from opencue_proto import report_pb2
from opencue_proto import service_pb2
from opencue_proto import show_pb2
from opencue_proto import subscription_pb2
from opencue_proto import task_pb2
from .cuebot import Cuebot
# pylint: disable=cyclic-import
from .wrappers.allocation import Allocation
from .wrappers.comment import Comment
from .wrappers.depend import Depend
from .wrappers.filter import Action
from .wrappers.filter import Filter
from .wrappers.filter import Matcher
from .wrappers.frame import Frame
from .wrappers.group import Group
from .wrappers.host import Host, NestedHost
from .wrappers.job import Job
from .wrappers.layer import Layer
from .wrappers.limit import Limit
from .wrappers.owner import Owner
from .wrappers.proc import Proc
from .wrappers.service import Service
from .wrappers.show import Show
from .wrappers.subscription import Subscription
from .wrappers.task import Task
from . import search
from . import util


__protobufs = [comment_pb2, criterion_pb2, cue_pb2, department_pb2, depend_pb2, facility_pb2,
               filter_pb2, host_pb2, job_pb2, renderPartition_pb2, report_pb2, service_pb2,
               show_pb2, subscription_pb2, task_pb2]

__wrappers = [Action, Allocation, Comment, Depend, Filter, Frame, Group, Host, Job, Layer, Matcher,
              NestedHost, Proc, Show, Subscription, Task]


#
# These are convenience methods that get imported into
# the package namespace.
#
@util.grpcExceptionParser
def getDefaultServices():
    """
    Return the default service list.  Services
    define the default application features.

    :rtype: list
    :return: List of Service objects
    """
    return Service.getDefaultServices()


@util.grpcExceptionParser
def getService(name):
    """
    Return the service with the provided name.

    :type name: str
    :param name: the name of the service
    :rtype: Service
    """
    return Service.getService(name)


@util.grpcExceptionParser
def createService(data):
    """
    Create the provided service and return it.

    :type data: Service
    :param data: Service object to create
    :rtype: Service
    """
    return Service(data).create()


@util.grpcExceptionParser
def getSystemStats():
    """Returns the system stats for a random
    OpenCue server in the cluster. This is used
    mainly by admins for troubleshooting OpenCue
    problems.

    :rtype: SystemStats
    :return: a struct of OpenCue application information."""
    return Cuebot.getStub('cue').GetSystemStats(
        cue_pb2.CueGetSystemStatsRequest(), timeout=Cuebot.Timeout).stats


#
# Facility
#
@util.grpcExceptionParser
def createFacility(name):
    """Create a given facility by name or unique ID.

    :type name: str
    :param name: a facility name or unique ID
    :rtype: Facility
    :return: a facility object
    """
    return Cuebot.getStub('facility').Create(
        facility_pb2.FacilityCreateRequest(name=name), timeout=Cuebot.Timeout).facility


@util.grpcExceptionParser
def getFacility(name):
    """Return a given facility by name or unique ID.

    :type name: str
    :param name: a facility name or unique ID
    :rtype: Facility
    :return: a facility object
    """
    return Cuebot.getStub('facility').Get(
        facility_pb2.FacilityGetRequest(name=name), timeout=Cuebot.Timeout).facility


@util.grpcExceptionParser
def renameFacility(facility, new_name):
    """Rename a given facility by name or unique ID.

    :type facility: str
    :param facility: an existing facility name or unique ID
    :type new_name: str
    :param new_name: a new facility name or unique ID
    """
    Cuebot.getStub('facility').Rename(
        facility_pb2.FacilityRenameRequest(facility=facility, new_name=new_name),
        timeout=Cuebot.Timeout)


@util.grpcExceptionParser
def deleteFacility(name):
    """Delete a given facility by name or unique ID.

    :type name: str
    :param name: a facility name or unique ID
    """
    Cuebot.getStub('facility').Delete(
        facility_pb2.FacilityDeleteRequest(name=name), timeout=Cuebot.Timeout)


#
# Departments
#
@util.grpcExceptionParser
def getDepartmentNames():
    """Return a list of the known department names.

    :rtype: list
    :return: a list of str department names
    """
    return list(Cuebot.getStub('department').GetDepartmentNames(
        department_pb2.DeptGetDepartmentNamesRequest(), timeout=Cuebot.Timeout).names)


#
# Shows
#
@util.grpcExceptionParser
def createShow(show):
    """Creates a new show.

     :type  show: str
     :param show: a new show name to create
     :rtype:  Show
     :return: the created show object"""
    return Show(Cuebot.getStub('show').CreateShow(
        show_pb2.ShowCreateShowRequest(name=show), timeout=Cuebot.Timeout).show)


@util.grpcExceptionParser
def deleteShow(show_id):
    """Deletes a show.

     :type  show_id: str
     :param show_id: a show ID to delete"""
    show = findShow(show_id)
    Cuebot.getStub('show').Delete(
        show_pb2.ShowDeleteRequest(show=show.data), timeout=Cuebot.Timeout)


@util.grpcExceptionParser
def getShows():
    """Returns a list of show objects.

    :rtype:  list
    :return: a list of Show objects"""
    showSeq = Cuebot.getStub('show').GetShows(
        show_pb2.ShowGetShowsRequest(), timeout=Cuebot.Timeout).shows
    return [Show(s) for s in showSeq.shows]


@util.grpcExceptionParser
def getActiveShows():
    """Returns a list of all active shows.

    :rtype:  list
    :return: a list of Show objects"""
    showSeq = Cuebot.getStub('show').GetActiveShows(
        show_pb2.ShowGetActiveShowsRequest(), timeout=Cuebot.Timeout).shows
    return [Show(s) for s in showSeq.shows]


@util.grpcExceptionParser
def findShow(name):
    """Returns a list of show objects.

    :type  name: str
    :param name: a string that represents a show to return
    :rtype:  Show
    :return: the matching Show object"""
    return Show(Cuebot.getStub('show').FindShow(
        show_pb2.ShowFindShowRequest(name=name), timeout=Cuebot.Timeout).show)


#
# Groups
#
@util.grpcExceptionParser
def findGroup(show, group):
    """Returns a group object.

    :type  show: str
    :param show: the name of a show
    :type  group: str
    :param group: the name of a group
    :rtype:  Group
    :return: the matching group object"""
    return Group(Cuebot.getStub('group').FindGroup(
        job_pb2.GroupFindGroupRequest(show=show, name=group), timeout=Cuebot.Timeout).group)


@util.grpcExceptionParser
def getGroup(uniq):
    """Returns a Group object from its unique ID.

    :type  uniq: str
    :param uniq: a unique group identifier
    :rtype:  Group
    :return: the matching group object"""
    return Group(Cuebot.getStub('group').GetGroup(
        job_pb2.GroupGetGroupRequest(id=uniq), timeout=Cuebot.Timeout).group)


#
# Jobs
#
@util.grpcExceptionParser
def findJob(name):
    """Returns a Job object for the given job name.
    This will only return one or zero active job.

    :type  name: str
    :param name: a job name
    :rtype:  Job
    :return: a Job object"""
    return Job(Cuebot.getStub('job').FindJob(
        job_pb2.JobFindJobRequest(name=name), timeout=Cuebot.Timeout).job)


@util.grpcExceptionParser
def getJob(uniq):
    """Returns a Job object for the given job ID.
    This will only return one or zero active job.

    :type  uniq: str
    :param uniq: a unique job identifier
    :rtype:  Job
    :return: a Job object"""
    return Job(Cuebot.getStub('job').GetJob(
        job_pb2.JobGetJobRequest(id=uniq), timeout=Cuebot.Timeout).job)


@util.grpcExceptionParser
def getJobs(**options):
    """
    Returns an array of Job objects using
    optional search criteria. Search criteria is
    supplied as a variable list of arguments.

    For example::

        # returns only pipe jobs.
        getJobs(show=["pipe"])

    Possible args:
        - job: job names - list
        - match:  job name substring match - str
        - regex: a job name search by regular expression - str
        - id: a job search by unique id - str
        - show: show names - list
        - shot: shot names - list
        - user: user names - list
        - include_finished - bool

    :rtype:  list
    :return: a list of Job objects
    """
    criteria = search.JobSearch.criteriaFromOptions(**options)
    jobSeq = Cuebot.getStub('job').GetJobs(
        job_pb2.JobGetJobsRequest(r=criteria), timeout=Cuebot.Timeout).jobs
    return [Job(j) for j in jobSeq.jobs]


@util.grpcExceptionParser
def isJobPending(name):
    """Returns true if there is an active job in the cue
    in the pending state.

    :type  name: str
    :param name: a job name
    :rtype: bool
    :return: true if the job exists"""
    return Cuebot.getStub('job').IsJobPending(
        job_pb2.JobIsJobPendingRequest(name=name), timeout=Cuebot.Timeout).value


@util.grpcExceptionParser
def launchSpec(spec):
    """Launch a new job with the given spec xml data.
    This call returns immediately but there is guarantee that
    the job was written to the database.

    :type spec: str
    :param spec: XML string containing job spec
    :rtype: list
    :return: List of str job names that were submitted
    """
    return Cuebot.getStub('job').LaunchSpec(
        job_pb2.JobLaunchSpecRequest(spec=spec), timeout=Cuebot.Timeout).names


@util.grpcExceptionParser
def launchSpecAndWait(spec):
    """Launch a new job with the given spec xml data.
    This call waits on the server until the job is committed
    in the database.

    :type spec: str
    :param spec: XML string containing job spec
    :rtype: list
    :return: List of Job objects that were submitted
    """
    jobSeq = Cuebot.getStub('job').LaunchSpecAndWait(
        job_pb2.JobLaunchSpecAndWaitRequest(spec=spec), timeout=Cuebot.Timeout).jobs
    return [Job(j) for j in jobSeq.jobs]


#
# Job Names
#
@util.grpcExceptionParser
def getJobNames(**options):
    """Returns a list of job names that match the search parameters.
    See getJobs for the job query options.

    :rtype:  list
    :return: List of matching str job names"""
    criteria = search.JobSearch.criteriaFromOptions(**options)
    return Cuebot.getStub('job').GetJobNames(
        job_pb2.JobGetJobNamesRequest(r=criteria), timeout=Cuebot.Timeout).names


#
# Layers
#
@util.grpcExceptionParser
def findLayer(job, layer):
    """Finds and returns a layer from the specified pending job.

    :type job: str
    :param job: the job name
    :type layer: str
    :param layer: the layer name
    :rtype: opencue.wrappers.layer.Layer
    :return: the layer matching the query"""
    return Layer(Cuebot.getStub('layer').FindLayer(
        job_pb2.LayerFindLayerRequest(job=job, layer=layer), timeout=Cuebot.Timeout).layer)


@util.grpcExceptionParser
def getLayer(uniq):
    """Returns a Layer object for the given layer ID.

    :type  uniq: str
    :param uniq: a unique layer identifier
    :rtype:  opencue.wrappers.layer.Layer
    :return: a Layer object"""
    return Layer(Cuebot.getStub('layer').GetLayer(
        job_pb2.LayerGetLayerRequest(id=uniq), timeout=Cuebot.Timeout).layer)


#
# Frames
#
@util.grpcExceptionParser
def findFrame(job, layer, number):
    """Finds and returns a layer from the specified pending job.

    :type job: str
    :param job: the job name
    :type layer: str
    :param layer: the layer name
    :type number: int
    :param number: the frame number
    :rtype: opencue.wrappers.frame.Frame
    :return: the frame matching the query"""
    return Frame(Cuebot.getStub('frame').FindFrame(
        job_pb2.FrameFindFrameRequest(job=job, layer=layer, frame=number),
        timeout=Cuebot.Timeout).frame)


@util.grpcExceptionParser
def getFrame(uniq):
    """Returns a Frame object from the unique ID.

    :type  uniq: str
    :param uniq: a unique frame identifier
    :rtype:  opencue.wrappers.frame.Frame
    :return: a Frame object"""
    return Frame(Cuebot.getStub('frame').GetFrame(
        job_pb2.FrameGetFrameRequest(id=uniq), timeout=Cuebot.Timeout).frame)


@util.grpcExceptionParser
def getFrames(job, **options):
    """Finds frames in a job that match the search criteria.

    :type job: str
    :param job: the job name
    :rtype: list
    :return: a list of matching Frame objects"""
    criteria = search.FrameSearch.criteriaFromOptions(**options)
    framesSeq = Cuebot.getStub('frame').GetFrames(
        job_pb2.FrameGetFramesRequest(job=job, r=criteria), timeout=Cuebot.Timeout).frames
    return [Frame(f) for f in framesSeq.frames]


#
# Depends
#
@util.grpcExceptionParser
def getDepend(uniq):
    """Finds a dependency from its unique ID.

    :type id: str
    :param id: the unique ID of the Depend object
    :rtype: opencue.wrappers.depend.Depend
    :return: a dependency"""
    return Depend(Cuebot.getStub('depend').GetDepend(
        depend_pb2.DependGetDependRequest(id=uniq), timeout=Cuebot.Timeout).depend)


#
# Hosts
#
@util.grpcExceptionParser
def getHostWhiteboard():
    """
    :rtype:  list<Host>
    :return: NestedHost """
    nestedHostSeq = Cuebot.getStub('host').GetHostWhiteboard(
        host_pb2.HostGetHostWhiteboardRequest(),
        timeout=Cuebot.Timeout).nested_hosts
    return [NestedHost(nh) for nh in nestedHostSeq.nested_hosts]


@util.grpcExceptionParser
def getHosts(**options):
    """
    Returns an array of Hroc objects using
    optional search criteria. Search criteria is
    supplied as a variable list of arguments.

    For example::

       # returns all vrack procs
       getHosts(match=["vrack"])

    Possible args:
       - host: host names - list
       - match: host name substring match - str
       - regex: a host name search by regular expression - str
       - id: a search by unique id - str
       - alloc: search by allocation. - list

    :rtype:  list
    :return: a list of Host objects
    """
    return search.HostSearch.byOptions(**options)


@util.grpcExceptionParser
def findHost(name):
    """Returns the host for the matching hostname.

    :type  name: str
    :param name: the unique name of a host
    :rtype:  Host
    :return: The matching host object"""
    return Host(Cuebot.getStub('host').FindHost(
        host_pb2.HostFindHostRequest(name=name), timeout=Cuebot.Timeout).host)


@util.grpcExceptionParser
def getHost(uniq):
    """Returns a Host object from a unique identifier.

    :type  uniq: str
    :param uniq: a unique host identifier
    :rtype:  Host
    :return: A Host object"""
    return Host(Cuebot.getStub('host').GetHost(
        host_pb2.HostGetHostRequest(id=uniq), timeout=Cuebot.Timeout).host)


#
# Owners
#
@util.grpcExceptionParser
def getOwner(owner_id):
    """Return an Owner object from the ID or name.

    :type  owner_id: str
    :param owner_id: a unique owner identifier or name
    :rtype:  Owner
    :return: An Owner object"""
    return Owner(Cuebot.getStub('owner').GetOwner(
        host_pb2.OwnerGetOwnerRequest(name=owner_id), timeout=Cuebot.Timeout).owner)

#
# Filters
#
@util.grpcExceptionParser
def findFilter(show_name, filter_name):
    """Returns the matching filter (for testing).

    :type  show_name: str
    :param show_name: a show name
    :type  filter_name: str
    :param filter_name: a filter name
    :rtype:  Filter
    :return: the matching Filter object"""
    return Filter(Cuebot.getStub('filter').FindFilter(
        filter_pb2.FilterFindFilterRequest(show=show_name, name=filter_name),
        timeout=Cuebot.Timeout).filter)

#
# Allocation
#
@util.grpcExceptionParser
def createAllocation(name, tag, facility):
    """Creates and returns an allocation.
    The host tag will be the lowercase of the allocation name.

    :type  name: str
    :param name: the name of the allocation
    :type  tag: str
    :param tag: the tag for the allocation
    :rtype:  Allocation
    :return: the newly created Allocation object"""
    return Allocation(Cuebot.getStub('allocation').Create(
        facility_pb2.AllocCreateRequest(name=name, tag=tag, facility=facility),
        timeout=Cuebot.Timeout).allocation)


@util.grpcExceptionParser
def getAllocations():
    """Returns a list of allocation objects.

    :rtype:  list
    :return: a list of Allocation objects"""
    allocationSeq = Cuebot.getStub('allocation').GetAll(
        facility_pb2.AllocGetAllRequest(), timeout=Cuebot.Timeout).allocations
    return [Allocation(a) for a in allocationSeq.allocations]


@util.grpcExceptionParser
def findAllocation(name):
    """Returns the Allocation object that matches the name.

    :type  name: str
    :param name: fully qualified name of the allocation (facility.allocation)
    :rtype:  Allocation
    :return: an Allocation object"""
    return Allocation(Cuebot.getStub('allocation').Find(
        facility_pb2.AllocFindRequest(name=name), timeout=Cuebot.Timeout).allocation)


@util.grpcExceptionParser
def getAllocation(allocId):
    """Returns the Allocation object that matches the ID.

    :type  allocId: str
    :param allocId: the ID of the allocation
    :rtype:  Allocation
    :return: an Allocation object"""
    return Allocation(Cuebot.getStub('allocation').Get(
        facility_pb2.AllocGetRequest(id=allocId), timeout=Cuebot.Timeout).allocation)


@util.grpcExceptionParser
def deleteAllocation(alloc):
    """Deletes an allocation.

    :type  alloc: facility_pb2.Allocation
    :param alloc: allocation to delete
    :rtype:  facility_pb2.AllocDeleteResponse
    :return: empty response"""
    return Cuebot.getStub('allocation').Delete(
        facility_pb2.AllocDeleteRequest(allocation=alloc), timeout=Cuebot.Timeout)


@util.grpcExceptionParser
def getDefaultAllocation():
    """Get the default allocation.

    :rtype:  Allocation
    :return: an Allocation object"""
    return Allocation(Cuebot.getStub('allocation').GetDefault(
        facility_pb2.AllocGetDefaultRequest(), timeout=Cuebot.Timeout).allocation)


@util.grpcExceptionParser
def setDefaultAllocation(alloc):
    """Set the default allocation.

    :type  alloc: facility_pb2.Allocation
    :param alloc: allocation to set default
    :rtype:  facility_pb2.AllocSetDefaultResponse
    :return: empty response"""
    return Cuebot.getStub('allocation').SetDefault(
        facility_pb2.AllocSetDefaultRequest(allocation=alloc), timeout=Cuebot.Timeout)


@util.grpcExceptionParser
def allocSetBillable(alloc, is_billable):
    """Sets an allocation billable or not.

    :type  alloc: facility_pb2.Allocation
    :param alloc: allocation to set
    :type  is_billable: bool
    :param is_billable: whether alloc should be billable or not
    :rtype:  facility_pb2.AllocSetBillableResponse
    :return: empty response
    """
    alloc.name = alloc.name.split(".")[-1]
    return Cuebot.getStub('allocation').SetBillable(
        facility_pb2.AllocSetBillableRequest(allocation=alloc, value=is_billable),
        timeout=Cuebot.Timeout)


@util.grpcExceptionParser
def allocSetName(alloc, name):
    """Sets an allocation name.

    :type  alloc: facility_pb2.Allocation
    :param alloc: allocation to set
    :type  name: str
    :param name: new name for the allocation
    :rtype:  facility_pb2.AllocSetNameResponse
    :return: empty response"""
    return Cuebot.getStub('allocation').SetName(
        facility_pb2.AllocSetNameRequest(allocation=alloc, name=name), timeout=Cuebot.Timeout)


@util.grpcExceptionParser
def allocSetTag(alloc, tag):
    """Sets an allocation tag.

    :type  alloc: facility_pb2.Allocation
    :param alloc: allocation to tag
    :type  tag: str
    :param tag: new tag
    :rtype:  facility_pb2.AllocSetTagResponse
    :return: empty response"""
    return Cuebot.getStub('allocation').SetTag(
        facility_pb2.AllocSetTagRequest(allocation=alloc, tag=tag), timeout=Cuebot.Timeout)


#
# Subscriptions
#
@util.grpcExceptionParser
def getSubscription(uniq):
    """Returns a Subscription object from a unique identifier.

    :type  uniq: str
    :param uniq: a unique subscription identifier
    :rtype:  Subscription
    :return: a Subscription object"""
    return Subscription(Cuebot.getStub('subscription').Get(
        subscription_pb2.SubscriptionGetRequest(id=uniq), timeout=Cuebot.Timeout).subscription)

@util.grpcExceptionParser
def findSubscription(name):
    """Returns the subscription object that matches the name.

    :type  name: str
    :param name: the name of the subscription
    :rtype:  Subscription
    :return: a Subscription object"""
    return Subscription(Cuebot.getStub('subscription').Find(
        subscription_pb2.SubscriptionFindRequest(name=name), timeout=Cuebot.Timeout).subscription)

#
# Procs
#
@util.grpcExceptionParser
def getProcs(**options):
    """Returns an array of Proc objects using
    optional search criteria. Search criteria is
    supplied as a variable list of arguments.

    For example::

        # returns procs running pipe jobs
        getProcs(show=["pipe"])

    Possible args:
       - host: host names - list
       - jobs: job names - list
       - layer: layer names - list
       - show: show names - list
       - alloc: allocation names - list
       - memory: used memory in gigabytes - str
         - "gt5" is greater than 5 gigs
         - "lt5" is less than 5 gigs
         - "5-10" is range of 5 to 10 gigs
       - duration: run time in hours - str
         - "gt5" is greater than 5 hours
         - "lt5" is less than 5 hours
         - "5-10" is range of 5 to 10 hours

    :rtype:  list[opencue.wrapper.proc.Proc]
    :return: a list of Proc objects"""
    procSeq = search.ProcSearch.byOptions(**options).procs
    return [Proc(p) for p in procSeq.procs]

#
# Limits
#
@util.grpcExceptionParser
def createLimit(name, maxValue):
    """Create a new Limit with the given name and max value.

    :type name: str
    :param name: the name of the new Limit
    :type maxValue: int
    :param maxValue: the maximum number of running frames for this limit
    :rtype: opencue.wrappers.limit.Limit
    :return: the newly created Limit
    """
    return Limit(Cuebot.getStub('limit').Create(
        limit_pb2.LimitCreateRequest(name=name, max_value=maxValue), timeout=Cuebot.Timeout))

@util.grpcExceptionParser
def getLimits():
    """Return a list of all known Limit objects.

    :rtype: list
    :return: a list of Limit objects"""
    return [Limit(limit) for limit in Cuebot.getStub('limit').GetAll(
        limit_pb2.LimitGetAllRequest(), timeout=Cuebot.Timeout).limits]

@util.grpcExceptionParser
def findLimit(name):
    """Returns the Limit object that matches the name.

    :type  name: str
    :param name: a string that represents a limit to return
    :rtype:  Limit
    :return: the matching Limit object"""
    return Limit(Cuebot.getStub('limit').Find(
        limit_pb2.LimitFindRequest(name=name), timeout=Cuebot.Timeout).limit)



#!/usr/bin/env python

#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for the opencue.wrappers.show module."""

from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest

import mock

from opencue_proto import facility_pb2
from opencue_proto import filter_pb2
from opencue_proto import host_pb2
from opencue_proto import job_pb2
from opencue_proto import service_pb2
from opencue_proto import show_pb2
from opencue_proto import subscription_pb2
import opencue.wrappers.allocation
import opencue.wrappers.show


TEST_ALLOCATION_ID = 'aaa-zzz-fff'
TEST_SHOW_NAME = 'pipe'
TEST_OWNER_NAME = 'opencue'
TEST_FILTER_NAME = 'unittest_filter'
TEST_SERVICE_NAME = 'unittest_service'
TEST_SUBSCRIPTION_NAME = 'unittest_subscription'
TEST_SUBSCRIPTION_SIZE = 1000
TEST_SUBSCRIPTION_BURST = 1200
TEST_MIN_CORES = 42
TEST_MAX_CORES = 47
TEST_MIN_GPUS = 2
TEST_MAX_GPUS = 7
TEST_ENABLE_VALUE = False
TEST_GROUP_NAME = 'group'
TEST_GROUP_DEPT = 'lighting'
TEST_TARGET_SHOW_NAME = 'target_show'


@mock.patch('opencue.cuebot.Cuebot.getStub')
class ShowTests(unittest.TestCase):

    def testCreateOwner(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.CreateOwner.return_value = show_pb2.ShowCreateOwnerResponse(
            owner=host_pb2.Owner(name=TEST_OWNER_NAME))
        getStubMock.return_value = stubMock

        show = opencue.wrappers.show.Show(show_pb2.Show(name=TEST_SHOW_NAME))
        owner = show.createOwner(TEST_OWNER_NAME)

        stubMock.CreateOwner.assert_called_with(
            show_pb2.ShowCreateOwnerRequest(show=show.data, name=TEST_OWNER_NAME),
            timeout=mock.ANY)
        self.assertEqual(owner.name, TEST_OWNER_NAME)

    def testCreateSubscription(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.CreateSubscription.return_value = show_pb2.ShowCreateSubscriptionResponse(
            subscription=subscription_pb2.Subscription(name=TEST_SUBSCRIPTION_NAME,
                                                       size=TEST_SUBSCRIPTION_SIZE,
                                                       burst=TEST_SUBSCRIPTION_BURST))
        getStubMock.return_value = stubMock

        show = opencue.wrappers.show.Show(show_pb2.Show(name=TEST_SHOW_NAME))
        allocation = opencue.wrappers.allocation.Allocation(
            facility_pb2.Allocation(id=TEST_ALLOCATION_ID))
        subscription = show.createSubscription(
            allocation, TEST_SUBSCRIPTION_SIZE, TEST_SUBSCRIPTION_BURST)

        stubMock.CreateSubscription.assert_called_with(
            show_pb2.ShowCreateSubscriptionRequest(show=show.data,
                                                   allocation_id=TEST_ALLOCATION_ID,
                                                   size=TEST_SUBSCRIPTION_SIZE,
                                                   burst=TEST_SUBSCRIPTION_BURST),
            timeout=mock.ANY)
        self.assertEqual(subscription.size(), TEST_SUBSCRIPTION_SIZE)
        self.assertEqual(subscription.burst(), TEST_SUBSCRIPTION_BURST)

    def testDelete(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Delete.return_value = show_pb2.ShowDeleteResponse()
        getStubMock.return_value = stubMock

        show = opencue.wrappers.show.Show(show_pb2.Show(name=TEST_SHOW_NAME))
        show.delete()

        stubMock.Delete.assert_called_with(
            show_pb2.ShowDeleteRequest(show=show.data),
            timeout=mock.ANY)

    def testArchive(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Archive.return_value = show_pb2.ShowArchiveResponse()
        getStubMock.return_value = stubMock

        show = opencue.wrappers.show.Show(show_pb2.Show(name=TEST_SHOW_NAME))
        show.archive(TEST_TARGET_SHOW_NAME)

        stubMock.Archive.assert_called_with(
            show_pb2.ShowArchiveRequest(show=show.data, target_show_name=TEST_TARGET_SHOW_NAME),
            timeout=mock.ANY)

    def testGetServiceOverrides(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetServiceOverrides.return_value = show_pb2.ShowGetServiceOverridesResponse(
            service_overrides=service_pb2.ServiceOverrideSeq(
                service_overrides=[
                    service_pb2.ServiceOverride(data=service_pb2.Service(name=TEST_SERVICE_NAME))
                ]))
        getStubMock.return_value = stubMock

        show = opencue.wrappers.show.Show(show_pb2.Show(name=TEST_SHOW_NAME))
        overrides = show.getServiceOverrides()

        stubMock.GetServiceOverrides.assert_called_with(
            show_pb2.ShowGetServiceOverridesRequest(show=show.data),
            timeout=mock.ANY)
        self.assertEqual(overrides[0].data.name, TEST_SERVICE_NAME)

    def testGetSubscriptions(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetSubscriptions.return_value = show_pb2.ShowGetSubscriptionResponse(
            subscriptions=subscription_pb2.SubscriptionSeq(
                subscriptions=[subscription_pb2.Subscription(name=TEST_SUBSCRIPTION_NAME)])
        )
        getStubMock.return_value = stubMock

        show = opencue.wrappers.show.Show(show_pb2.Show(name=TEST_SHOW_NAME))
        subscriptions = show.getSubscriptions()

        stubMock.GetSubscriptions.assert_called_with(
            show_pb2.ShowGetSubscriptionRequest(show=show.data), timeout=mock.ANY)
        self.assertEqual(len(subscriptions), 1)
        self.assertEqual(subscriptions[0].name(), TEST_SUBSCRIPTION_NAME)

    def testFindSubscription(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Find.return_value = subscription_pb2.SubscriptionFindResponse(
            subscription=subscription_pb2.Subscription(name=TEST_SUBSCRIPTION_NAME))
        getStubMock.return_value = stubMock

        show = opencue.wrappers.show.Show(show_pb2.Show(name=TEST_SHOW_NAME))
        subscription = show.findSubscription(TEST_SUBSCRIPTION_NAME)

        stubMock.Find.assert_called_with(
            subscription_pb2.SubscriptionFindRequest(name=TEST_SUBSCRIPTION_NAME), timeout=mock.ANY)
        self.assertEqual(subscription.name(), TEST_SUBSCRIPTION_NAME)

    def testGetFilters(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetFilters.return_value = show_pb2.ShowGetFiltersResponse(
            filters=filter_pb2.FilterSeq(filters=[filter_pb2.Filter(name=TEST_FILTER_NAME)]))
        getStubMock.return_value = stubMock

        show = opencue.wrappers.show.Show(show_pb2.Show(name=TEST_SHOW_NAME))
        filters = show.getFilters()

        stubMock.GetFilters.assert_called_with(
            show_pb2.ShowGetFiltersRequest(show=show.data), timeout=mock.ANY)
        self.assertEqual(len(filters), 1)
        self.assertEqual(filters[0].name(), TEST_FILTER_NAME)

    def testSetActive(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetActive.return_value = show_pb2.ShowSetActiveResponse()
        getStubMock.return_value = stubMock

        show = opencue.wrappers.show.Show(show_pb2.Show(name=TEST_SHOW_NAME))
        show.setActive(TEST_ENABLE_VALUE)

        stubMock.SetActive.assert_called_with(
            show_pb2.ShowSetActiveRequest(show=show.data, value=TEST_ENABLE_VALUE),
            timeout=mock.ANY)

    def testSetDefaultMaxCores(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetDefaultMaxCores.return_value = show_pb2.ShowSetDefaultMaxCoresResponse()
        getStubMock.return_value = stubMock

        show = opencue.wrappers.show.Show(show_pb2.Show(name=TEST_SHOW_NAME))
        show.setDefaultMaxCores(TEST_MAX_CORES)

        stubMock.SetDefaultMaxCores.assert_called_with(
            show_pb2.ShowSetDefaultMaxCoresRequest(show=show.data, max_cores=TEST_MAX_CORES),
            timeout=mock.ANY)

    def testSetDefaultMinCores(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetDefaultMinCores.return_value = show_pb2.ShowSetDefaultMinCoresResponse()
        getStubMock.return_value = stubMock

        show = opencue.wrappers.show.Show(show_pb2.Show(name=TEST_SHOW_NAME))
        show.setDefaultMinCores(TEST_MIN_CORES)

        stubMock.SetDefaultMinCores.assert_called_with(
            show_pb2.ShowSetDefaultMinCoresRequest(show=show.data, min_cores=TEST_MIN_CORES),
            timeout=mock.ANY)

    def testSetDefaultMaxGpus(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetDefaultMaxGpus.return_value = show_pb2.ShowSetDefaultMaxGpusResponse()
        getStubMock.return_value = stubMock

        show = opencue.wrappers.show.Show(show_pb2.Show(name=TEST_SHOW_NAME))
        show.setDefaultMaxGpus(TEST_MAX_GPUS)

        stubMock.SetDefaultMaxGpus.assert_called_with(
            show_pb2.ShowSetDefaultMaxGpusRequest(show=show.data, max_gpus=TEST_MAX_GPUS),
            timeout=mock.ANY)

    def testSetDefaultMinGpus(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.SetDefaultMinGpus.return_value = show_pb2.ShowSetDefaultMinGpusResponse()
        getStubMock.return_value = stubMock

        show = opencue.wrappers.show.Show(show_pb2.Show(name=TEST_SHOW_NAME))
        show.setDefaultMinGpus(TEST_MIN_GPUS)

        stubMock.SetDefaultMinGpus.assert_called_with(
            show_pb2.ShowSetDefaultMinGpusRequest(show=show.data, min_gpus=TEST_MIN_GPUS),
            timeout=mock.ANY)

    def testFindFilter(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.FindFilter.return_value = show_pb2.ShowFindFilterResponse(
            filter=filter_pb2.Filter(name=TEST_FILTER_NAME))
        getStubMock.return_value = stubMock

        show = opencue.wrappers.show.Show(show_pb2.Show(name=TEST_SHOW_NAME))
        filter_found = show.findFilter(TEST_FILTER_NAME)

        stubMock.FindFilter.assert_called_with(
            show_pb2.ShowFindFilterRequest(show=show.data, name=TEST_FILTER_NAME),
            timeout=mock.ANY)
        self.assertEqual(filter_found.name(), TEST_FILTER_NAME)

    def testCreateFilter(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.CreateFilter.return_value = show_pb2.ShowCreateFilterResponse(
            filter=filter_pb2.Filter(name=TEST_FILTER_NAME))
        getStubMock.return_value = stubMock

        show = opencue.wrappers.show.Show(show_pb2.Show(name=TEST_SHOW_NAME))
        filter_created = show.createFilter(TEST_FILTER_NAME)

        stubMock.CreateFilter.assert_called_with(
            show_pb2.ShowCreateFilterRequest(show=show.data, name=TEST_FILTER_NAME),
            timeout=mock.ANY)
        self.assertEqual(filter_created.name(), TEST_FILTER_NAME)

    def testGetGroups(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetGroups.return_value = show_pb2.ShowGetGroupsResponse(
            groups=job_pb2.GroupSeq(
                groups=[job_pb2.Group(name=TEST_GROUP_NAME, department=TEST_GROUP_DEPT)])
        )
        getStubMock.return_value = stubMock

        show = opencue.wrappers.show.Show(show_pb2.Show(name=TEST_SHOW_NAME))
        groups = show.getGroups()

        stubMock.GetGroups.assert_called_with(
            show_pb2.ShowGetGroupsRequest(show=show.data), timeout=mock.ANY)
        self.assertEqual(len(groups), 1)
        self.assertEqual(groups[0].name(), TEST_GROUP_NAME)
        self.assertEqual(groups[0].department(), TEST_GROUP_DEPT)

    def testGetJobWhiteboard(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetJobWhiteboard.return_value = show_pb2.ShowGetJobWhiteboardResponse(
            whiteboard=job_pb2.NestedGroup(name=TEST_GROUP_NAME,
                                           department=TEST_GROUP_DEPT)
        )
        getStubMock.return_value = stubMock

        show = opencue.wrappers.show.Show(show_pb2.Show(name=TEST_SHOW_NAME))
        whiteboard = show.getJobWhiteboard()

        stubMock.GetJobWhiteboard.assert_called_with(
            show_pb2.ShowGetJobWhiteboardRequest(show=show.data), timeout=mock.ANY)
        self.assertEqual(whiteboard.name, TEST_GROUP_NAME)
        self.assertEqual(whiteboard.department, TEST_GROUP_DEPT)

    def testGetRootGroup(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetRootGroup.return_value = show_pb2.ShowGetRootGroupResponse(
            group=job_pb2.Group(name=TEST_GROUP_NAME,
                                department=TEST_GROUP_DEPT)
        )
        getStubMock.return_value = stubMock

        show = opencue.wrappers.show.Show(show_pb2.Show(name=TEST_SHOW_NAME))
        rootGroup = show.getRootGroup()

        stubMock.GetRootGroup.assert_called_with(
            show_pb2.ShowGetRootGroupRequest(show=show.data), timeout=mock.ANY)
        self.assertEqual(rootGroup.name(), TEST_GROUP_NAME)
        self.assertEqual(rootGroup.department(), TEST_GROUP_DEPT)

    def testEnableBooking(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.EnableBooking.return_value = show_pb2.ShowEnableBookingResponse()
        getStubMock.return_value = stubMock

        show = opencue.wrappers.show.Show(show_pb2.Show(name=TEST_SHOW_NAME))
        show.enableBooking(TEST_ENABLE_VALUE)

        stubMock.EnableBooking.assert_called_with(
            show_pb2.ShowEnableBookingRequest(show=show.data, enabled=TEST_ENABLE_VALUE),
            timeout=mock.ANY)

    def testEnableDispatching(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.EnableDispatching.return_value = show_pb2.ShowEnableDispatchingResponse()
        getStubMock.return_value = stubMock

        show = opencue.wrappers.show.Show(show_pb2.Show(name=TEST_SHOW_NAME))
        show.enableDispatching(TEST_ENABLE_VALUE)

        stubMock.EnableDispatching.assert_called_with(
            show_pb2.ShowEnableDispatchingRequest(show=show.data, enabled=TEST_ENABLE_VALUE),
            timeout=mock.ANY)

    def testCreateServiceOverrideMemError(self, getStubMock):
        service = service_pb2.Service(name=TEST_SERVICE_NAME)
        show = opencue.wrappers.show.Show(show_pb2.Show(name=TEST_SHOW_NAME))

        self.assertRaises(ValueError, show.createServiceOverride, service)


if __name__ == '__main__':
    unittest.main()



#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Utility methods used by the wrapper classes."""

import time


# pylint: disable=redefined-builtin
def format_time(epoch, format="%m/%d %H:%M", default="--/-- --:--"):
    """Formats time using time formatting standards.

    See: https://docs.python.org/3/library/time.html

    :type  epoch: int
    :param epoch: time as an epoch
    :type  format: str
    :param format: desired format of output string
    :type  default: str
    :param default: the output if the given time is empty
    :rtype: str
    :return: string-formatted version of the given time
    """
    if not epoch:
        return default
    return time.strftime(format, time.localtime(epoch))


def dateToMMDDHHMM(sec):
    """Returns a time in the format `%m/%d %H:%M`.

    :type  sec: int
    :param sec: time as an epoch
    :rtype:  str
    :return: time in the format %m/%d %H:%M
    """
    if sec == 0:
        return "--/-- --:--"
    return time.strftime("%m/%d %H:%M", time.localtime(sec))


def __splitTime(sec):
    """Splits a timestamp into hour, minute, and second components.

    :type  sec: int
    :param sec: timestamp as an epoch
    :rtype:  tuple
    :return: (hour, min, sec)
    """
    minute, sec = divmod(sec, 60)
    hour, minute = divmod(minute, 60)
    return hour, minute, sec


def secondsToHHMMSS(sec):
    """Returns time in the format HH:MM:SS

    :rtype:  str
    :return: Time in the format HH:MM:SS"""
    return "%02d:%02d:%02d" % __splitTime(sec)


def secondsToHMMSS(sec):
    """Returns time in the format H:MM:SS

    :rtype:  str
    :return: Time in the format H:MM:SS"""
    return "%d:%02d:%02d" % __splitTime(sec)


def secondsToHHHMM(sec):
    """Returns time in the format HHH:MM

    :rtype:  str
    :return: Time in the format HHH:MM"""
    return "%03d:%02d" % __splitTime(sec)[:2]


def secondsDiffToHMMSS(secA, secB):
    """Returns time difference of arguments in the format H:MM:SS

    :type  secA: int or float
    :param secA: seconds. 0 will be replaced with current time
    :type  secB: int or float
    :param secB: seconds. 0 will be replaced with current time
    :rtype:  str
    :return: Time difference of arguments in the format H:MM:SS
    """
    if secA == 0:
        secA = time.time()
    if secB == 0:
        secB = time.time()
    return secondsToHMMSS(max(secA, secB) - min(secA, secB))


def convert_mem(kmem, unit=None):
    """Returns an amount of memory in a human-readable string.

    :type  kmem: int
    :param kmem: amount of memory in kB
    :rtype:  str
    :return: same amount of memory formatted into a human-readable string
    """
    k = 1024
    if unit == 'K' or (unit is None and kmem < k):
        return '%dK' % kmem
    if unit == 'M' or (unit is None and kmem < pow(k, 2)):
        return '%dM' % (kmem / k)
    if unit == 'G' or (unit is None and kmem < pow(k, 3)):
        return '%.01fG' % (float(kmem) / pow(k, 2))
    return str(kmem)



#!/usr/bin/env python

#  Copyright Contributors to the OpenCue Project
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for `opencue.wrappers.service`."""

from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import unittest

import mock

from opencue_proto import service_pb2
import opencue.wrappers.service


TEST_SERVICE_NAME = 'testService'
TEST_MIN_GPUS = 2
TEST_MAX_GPUS = 7
TEST_MIN_GPU_MEMORY = 8 * 1024 * 1024 * 1024


@mock.patch('opencue.cuebot.Cuebot.getStub')
class ServiceTests(unittest.TestCase):

    def testDelete(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Delete.return_value = service_pb2.ServiceDeleteResponse()
        getStubMock.return_value = stubMock

        service = opencue.wrappers.service.Service(
            service_pb2.Service(name=TEST_SERVICE_NAME))
        service.delete()

        stubMock.Delete.assert_called_with(
            service_pb2.ServiceDeleteRequest(service=service.data), timeout=mock.ANY)

    def testCreateService(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.CreateService.return_value = service_pb2.ServiceCreateServiceResponse(
            service=service_pb2.Service(name=TEST_SERVICE_NAME, min_memory_increase=2097152))
        getStubMock.return_value = stubMock

        wrapper = opencue.wrappers.service.Service(
            service_pb2.Service(name=TEST_SERVICE_NAME, min_memory_increase=2097152))
        service = wrapper.create()

        stubMock.CreateService.assert_called_with(
            service_pb2.ServiceCreateServiceRequest(data=wrapper.data), timeout=mock.ANY)
        self.assertEqual(wrapper.name(), service.name())
        self.assertEqual(wrapper.minMemoryIncrease(), service.minMemoryIncrease())

    def testCreateServiceMemError(self, getStubMock):
        service = opencue.wrappers.service.Service(service_pb2.Service(
            name=TEST_SERVICE_NAME))

        self.assertRaises(ValueError, service.create)

    def testGetDefaultServices(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetDefaultServices.return_value = service_pb2.ServiceGetDefaultServicesResponse(
            services=service_pb2.ServiceSeq(
                services=[service_pb2.Service(name=TEST_SERVICE_NAME)]))
        getStubMock.return_value = stubMock

        wrapper = opencue.wrappers.service.Service()
        services = wrapper.getDefaultServices()

        stubMock.GetDefaultServices.assert_called_with(
            service_pb2.ServiceGetDefaultServicesRequest(), timeout=mock.ANY)
        self.assertEqual(len(services), 1)
        self.assertEqual(services[0].name(), TEST_SERVICE_NAME)

    def testGetService(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetService.return_value = service_pb2.ServiceGetServiceResponse(
            service=service_pb2.Service(name=TEST_SERVICE_NAME))
        getStubMock.return_value = stubMock

        wrapper = opencue.wrappers.service.Service()
        service = wrapper.getService(name=TEST_SERVICE_NAME)

        stubMock.GetService.assert_called_with(
            service_pb2.ServiceGetServiceRequest(name=TEST_SERVICE_NAME), timeout=mock.ANY)
        self.assertEqual(service.name(), TEST_SERVICE_NAME)

    def testUpdate(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.Update.return_value = service_pb2.ServiceUpdateResponse()
        getStubMock.return_value = stubMock

        wrapper = opencue.wrappers.service.Service(service=service_pb2.Service(
            name=TEST_SERVICE_NAME, min_memory_increase=302))
        wrapper.update()

        stubMock.Update.assert_called_with(
            service_pb2.ServiceUpdateRequest(service=wrapper.data), timeout=mock.ANY)

    def testGpus(self, getStubMock):
        stubMock = mock.Mock()
        stubMock.GetService.return_value = service_pb2.ServiceGetServiceResponse(
            service=service_pb2.Service(name=TEST_SERVICE_NAME))
        getStubMock.return_value = stubMock

        wrapper = opencue.wrappers.service.Service()
        service = wrapper.getService(name=TEST_SERVICE_NAME)
        self.assertEqual(service.minGpus(), 0)
        self.assertEqual(service.maxGpus(), 0)
        self.assertEqual(service.minGpuMemory(), 0)
        service.setMinGpus(TEST_MIN_GPUS)
        service.setMaxGpus(TEST_MAX_GPUS)
        service.setMinGpuMemory(TEST_MIN_GPU_MEMORY)
        self.assertEqual(service.minGpus(), TEST_MIN_GPUS)
        self.assertEqual(service.maxGpus(), TEST_MAX_GPUS)
        self.assertEqual(service.minGpuMemory(), TEST_MIN_GPU_MEMORY)

        stubMock.GetService.assert_called_with(
            service_pb2.ServiceGetServiceRequest(name=TEST_SERVICE_NAME), timeout=mock.ANY)
        self.assertEqual(service.name(), TEST_SERVICE_NAME)

    def testUpdateMemError(self, getStubMock):
        service = opencue.wrappers.service.Service(service=service_pb2.Service(
            name=TEST_SERVICE_NAME))

        self.assertRaises(ValueError, service.update)

    def testSetMinMemIncrease(self, getStubMock):
        service = opencue.wrappers.service.Service(
            service_pb2.Service(name=TEST_SERVICE_NAME,
                                min_memory_increase=2097152))

        self.assertRaises(ValueError, service.setMinMemoryIncrease, -1)
        self.assertRaises(ValueError, service.setMinMemoryIncrease, 0)

        service.setMinMemoryIncrease(12345678)
        self.assertEqual(service.minMemoryIncrease(), 12345678)


if __name__ == '__main__':
    unittest.main()
